package net.wanji.opt.common.enums;

import cn.hutool.core.util.ObjectUtil;
import lombok.AllArgsConstructor;
import lombok.Getter;
import net.wanji.common.utils.tool.StringUtils;

/**
 * @author duanruiming
 * @date 2025/01/07 20:24
 */
@Getter
@AllArgsConstructor
public enum EventInfoTypeEnum {
    // 事件类型转化
    CROSS_SMOOTH(0, "700","路口畅通"),
    PHASE_EMPTY(5, "701","相位空放"),
    CROSS_UNBALANCE(1, "702","路口失衡"),
    CROSS_OVERFLOW(3, "703","路口溢出"),
    // 当前没有
    //CROSS_DEADLOCK(4, "704","路口死锁"),
    GREEN_SLOW(6, "705","干线缓行"),
    GREEN_CONGEST(7, "706","干线拥堵"),
    CROSS_CONGEST(2, "707","路口拥堵"),
    GREEN_SMOOTH(8, "708","干线畅通");

    private Integer optType;
    private String eventType;
    private String desc;

    /**
     * 通过事件类型获取优化类型
     * @param eventType
     * @return
     */
    public static Integer getOptType(String eventType) {
        for (EventInfoTypeEnum value : EventInfoTypeEnum.values()) {
            if (StringUtils.equalsIgnoreCase(eventType, value.getEventType())) {
                return value.getOptType();
            }
        }
        return null;
    }

    /**
     * 过事件类型获取优化类型名称
     * @param eventType
     * @return
     */
    public static String getOptDesc(String eventType) {
        for (EventInfoTypeEnum value : EventInfoTypeEnum.values()) {
            if (StringUtils.equalsIgnoreCase(eventType, value.getEventType())) {
                return value.getDesc();
            }
        }
        return null;
    }

    /**
     * 通过optType获取枚举值
     * @param optType
     * @return
     */
    public static EventInfoTypeEnum getEventInfoTypeEnum(int optType) {
        for (EventInfoTypeEnum value : EventInfoTypeEnum.values()) {
            if (ObjectUtil.equals(value.getOptType(),optType)){
                return value;
            }
        }
        return null;
    }
}
