package net.wanji.opt.common.exception;

import com.cvicse.bixi.connector.ClientAbortException;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.exceptionhandler.GlobalExceptionHandler;
import net.wanji.common.framework.rest.JsonViewObject;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * 全局异常处理
 *
 * @author Kent HAN
 * @date 2023/1/5 13:18
 */
@RestControllerAdvice
@Slf4j
public class OptExceptionHandler extends GlobalExceptionHandler {
    /**
     * 字段值重复异常
     */
    @ExceptionHandler(value = UniqueException.class)
    public JsonViewObject uniqueExceptionHandler(UniqueException e) {
        return JsonViewObject.newInstance().fail(e);
    }

    @ExceptionHandler(value = OptServiceException.class)
    public JsonViewObject OptServiceExceptionHandler(OptServiceException e) {
        return JsonViewObject.newInstance().fail(e.getMessage(), "optService服务调用异常");
    }

    @ExceptionHandler(value = ClientAbortException.class)
    @ResponseStatus(HttpStatus.OK)
    public JsonViewObject clientExceptionHandler(ClientAbortException ex) {
        log.error("ClientAbortException异常opt:{}", ex);
        return JsonViewObject.newInstance().success("");
    }
}