package net.wanji.opt.constant;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * @author wanji
 * @version 1.0
 * Created on 2024/12/2 15:07
 * @Description: [枚举类]
 */

public enum CrossEnum {

    //******************************************************//
            EXT_NS ("-100","延长南北方向直行的绿灯"),
            EXT_EW ("-101" ,"延长东西方向直行的绿灯"),
            EXT_NS_L ("-102" ,"延长南北方向左转的绿灯"),
            EXT_EW_L ("-103","延长东西方向左转的绿灯"),
            EXT_N2S ("-110","延长北向南直行的绿灯") ,
            EXT_E2W ("-112","延长东向西直行的绿灯") ,
            EXT_S2N ("-114","延长南向北直行的绿灯"),
            EXT_W2E ("-116","延长西向东直行的绿灯") ,
            EXT_N2S_L ("-120","延长北向南左转的绿灯"),
            EXT_E2W_L ("-122","延长东向西左转的绿灯"),
            EXT_S2N_L ("-124","延长南向北左转的绿灯"),
            EXT_W2E_L ("-126","延长西向东左转的绿灯"),
            EXT_CUR ("-150","延长当前方向的绿灯") ,
            SKIP_NS ("-200","南北方向直行绿灯步进到下一个相位"),
            SKIP_EW ("-201","东西方向直行绿灯步进到下一个相位"),
            SKIP_NS_L ("-202","南北方向左转绿灯步进到下一个相位"),
            SKIP_EW_L ("-203","东西方向左转绿灯步进到下一个相位"),
            SKIP_N2S ("-210","北向南直行绿灯步进到下一个相位"),
            SKIP_E2W ("-212","东向西直行绿灯步进到下一个相位"),
            SKIP_S2N ("-214","南向北直行绿灯步进到下一个相位"),
            SKIP_W2E ("-216","西向东直行绿灯步进到下一个相位"),
            SKIP_N2S_L ("-220","北向南左转绿灯步进到下一个相位"),
            SKIP_E2W_L ("-222","东向西左转绿灯步进到下一个相位"),
            SKIP_S2N_L ("-224","南向北左转绿灯步进到下一个相位"),
            SKIP_W2E_L ("-226","西向东左转绿灯步进到下一个相位"),
            SKIP_CUR ("-250","当前绿灯步进到下一个相位"),
            SKIP_E2W1 ("2","东北方向溢出，延长相位"),
            SKIP_S2N2 ("3","东方向溢出，延长相位"),
            SKIP_W2E3 ("4","东南方向溢出，延长相位"),
            SKIP_N2S_L4 ("5","南方向溢出，延长相位"),
            SKIP_E2W_L5 ("6","西南方向溢出，延长相位"),
            SKIP_S2N_L6 ("7","西方向溢出，延长相位"),
            SKIP_W2E_L7 ("8","西北方向溢出，延长相位"),
            SKIP_CUR0 ("1","北方向溢出，延长相位");

    //******************************************************//

    private String type;
    private String name;

    CrossEnum(String type, String name) {
        this.type = type;
        this.name = name;
    }

    public static String getName(String type) {
        for (CrossEnum typeEnum : CrossEnum.values()) {
            if (Objects.equals(typeEnum.type, type)) {
                return typeEnum.name;
            }
        }
        return null;
    }

    public static CrossEnum getTypeEnum(String type) {
        for (CrossEnum typeEnum : CrossEnum.values()) {
            if (Objects.equals(typeEnum.type, type)) {
                return typeEnum;
            }
        }
        return null;
    }

    public String getType() {
        return type;
    }

    public String getName() {
        return name;
    }

    //空放相位延长
    public static final List<CrossEnum> ExtendPhaseList = Arrays.asList(
            new CrossEnum[] {
                    CrossEnum.EXT_NS,
                    CrossEnum.EXT_EW,
                    CrossEnum.EXT_NS_L,
                    CrossEnum.EXT_EW_L,
                    CrossEnum.EXT_N2S,
                    CrossEnum.EXT_E2W,
                    CrossEnum.EXT_S2N,
                    CrossEnum.EXT_W2E,
                    CrossEnum.EXT_N2S_L,
                    CrossEnum.EXT_E2W_L,
                    CrossEnum.EXT_S2N_L,
                    CrossEnum.EXT_W2E_L,
                    CrossEnum.EXT_CUR
            }
    );

    /**
     * 是否为延长相位
     * @param crossEnum
     * @return
     */
    public static boolean isExtendPhase(CrossEnum crossEnum){
        return ExtendPhaseList.contains(crossEnum);
    }

    //空放相位切换
    public static final List<CrossEnum> SwitchPhaseList = Arrays.asList(
            new CrossEnum[] {
                    CrossEnum.SKIP_NS,
                    CrossEnum.SKIP_EW,
                    CrossEnum.SKIP_NS_L,
                    CrossEnum.SKIP_EW_L,
                    CrossEnum.SKIP_N2S,
                    CrossEnum.SKIP_E2W,
                    CrossEnum.SKIP_S2N,
                    CrossEnum.SKIP_W2E,
                    CrossEnum.SKIP_N2S_L,
                    CrossEnum.SKIP_E2W_L,
                    CrossEnum.SKIP_S2N_L,
                    CrossEnum.SKIP_W2E_L,
                    CrossEnum.SKIP_CUR
            }
    );

    /**
     * 是否为切换相位
     * @param crossEnum
     * @return
     */
    public static boolean isSwitchPhase(CrossEnum crossEnum){
        return SwitchPhaseList.contains(crossEnum);
    }

    public static void main(String[] args) {
        System.out.println(CrossEnum.getTypeEnum("-100").type);
    }
}
