package net.wanji.opt.dao.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import net.wanji.databus.dto.EventInfoTrafficStatusDTO;
import net.wanji.databus.po.EventInfoTrafficStatusPO;
import net.wanji.opt.dto.EventTypeTimeRateDTO;
import net.wanji.opt.entity.EventAlarmInfo;
import net.wanji.opt.po.base.EventStatisticPo;
import net.wanji.opt.po.trend.EventInfoSimplePo;
import net.wanji.opt.po.trend.HoloEventInfoPO;
import net.wanji.opt.vo.CrossEventListPO;
import net.wanji.opt.vo2.CrossGreenStatusTimeRateVO;
import net.wanji.opt.vo2.OptMonitoringVO;
import net.wanji.opt.vo2.StatisticsEventTypeCountTimeVO;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2023/01/31 18:32
 */
@Repository
public interface HoloEventMapper extends BaseMapper<HoloEventInfoPO> {
    Integer selectCrossEmergencyCount(String crossId, int startStamp, int endStamp);

    Integer selectEmergencyCountWithLaneIds(String crossId, int startStamp, int endStamp, List<String> laneIds);

    List<HoloEventInfoPO> selectAIList(Date startTime);

    List<HoloEventInfoPO> selectAIGreenList(Date startTime);
    List<CrossEventListPO> selectCrossEventList();

    EventStatisticPo findCrossEventCount(Map<String,Object> params);

    List<EventInfoSimplePo> findEventDistribute(Map<String,Object> params);

    List<EventInfoTrafficStatusPO> trafficStatus(EventInfoTrafficStatusDTO eventInfoTrafficStatusDTO);

    /**
     * 查询实时、历史报警记录
     * @param realtimeType
     * @param eventType
     * @param name
     * @return
     */
    List<EventAlarmInfo> findEventAlarmInfo(String realtimeType, String eventType, String name);

    /**
     * 查看路口为结束的报警状态
     * @return
     */
    List<EventAlarmInfo> findNotFinishAlarmInfo();

    /**
     * 查询事件表，事件类型持续时间与当前时间的时间比例，对应交通体检-交通状态
     * @param crossId
     * @return
     */
    List<EventTypeTimeRateDTO> selectEventTypeTimeRate(String crossId);

    /**
     * 态势监测-区域体检-雷达图 查询
     * @return
     */
    List<CrossGreenStatusTimeRateVO> selectCrossGreenStatusTimeRate();

    /**
     * 态势监测-运行监测-干线优化监测
     * @return
     */
    List<OptMonitoringVO> selectGreenOptMonitoringList();

    /**
     * 态势监测-运行监测-路口优化监测
     * @return
     */
    List<OptMonitoringVO> selectCrossOptMonitoringList();

    /**
     * 态势监测-区域体检-雷达图事件类型过滤
     * @return
     */
    List<StatisticsEventTypeCountTimeVO> selectGreenEventTypeCountTimeList(@Param("type") String type,
                                                                           @Param("startTime") LocalDateTime startTime,
                                                                           @Param("endTime") LocalDateTime endTime);

    /**
     * 态势监测-区域体检-雷达图事件类型过滤
     * @return
     */
    List<StatisticsEventTypeCountTimeVO> selectCrossEventTypeCountTimeList(@Param("type") String type,
                                                                           @Param("startTime") LocalDateTime startTime,
                                                                           @Param("endTime") LocalDateTime endTime);

    /**
     * 态势监测-事件告警-历史列表
     * @return
     */
    List<StatisticsEventTypeCountTimeVO> selectCrossGreenHistList();
}
