package net.wanji.opt.servicev2;

import net.wanji.opt.synthesis.pojo.CrossRealTimeAlarmEntity;
import net.wanji.opt.synthesis.pojo.CrossStatusDisOptTimeEntity;
import net.wanji.opt.synthesis.pojo.vo.CrossOrGreenWaveTypeEntity;
import net.wanji.opt.vo2.CrossBaseInfoVO;
import net.wanji.opt.vo2.CrossStatusDistributionVO;

import java.util.List;
import java.util.Map;

/**
 * <p>
 *     监测详情-路口事件详情
 * </p>
 * @author zhengyifan
 * @date 2025/3/15
 */
public interface CrossService {
    /**
     * 获取路口状态分布
     * @param crossID 路口ID
     * @param date 日期
     * @param groupType 时间粒度 参数要求：0--5分钟 1--15分钟 2--30分钟 3--60分钟
     * @param objectType 范围 参数要求：1--方向级 2--转向级 3--车道级 4--路口级
     * @param condition 筛选条件-根据范围参数确定
     * @return 包含状态分布信息列表与优化时间列表
     */
    Map<String, Object> getCrossStatusDistribution(String crossID, String date, String groupType, Integer objectType, String condition);

    /**
     * 获取路口状态分布_单状态
     *
     * @param crossID    路口ID
     * @param date       日期
     * @param groupType  时间粒度 参数要求：0--5分钟 1--15分钟 2--30分钟 3--60分钟
     * @param objectType 范围 参数要求：1--方向级 2--转向级 3--车道级 4--路口级
     * @param condition  筛选条件-根据范围参数确定
     * @return 包含状态分布信息列表与优化时间列表
     */
    List<CrossStatusDistributionVO> getCrossStatusDistribution_SingleCondition(String crossID, String date, String groupType, Integer objectType, String condition);


    /**
     * 获取路口实时告警
     * @param crossID 路口ID
     * @param time 可选参数
     * @return
     */
    List<CrossRealTimeAlarmEntity> getCrossRealTimeAlarm(String crossID, String time);

    /**
     * 获取路口优化时间
     * @param crossID 路口ID
     * @return
     */
    List<CrossStatusDisOptTimeEntity> getOptTimeList(String crossID);

    /**
     * 获取路口基础信息————方向、转向、车道信息
     */
    CrossBaseInfoVO getCrossBaseInfos(String crossID);

    /**
     * 获取路口类型信息
     *
     * @param crossID 路口ID
     * @param date    日期
     */
    List<CrossOrGreenWaveTypeEntity> getCrossTypeList(String crossID, String date);

}

