package net.wanji.opt.servicev2.evaluation.impl;


import com.alibaba.fastjson.JSONArray;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.dubbointerface.impl.BaseDubboInterfaceImpl;
import net.wanji.common.framework.mapper.BaseInterfaceMapper;
import net.wanji.opt.common.CommonUtils;
import net.wanji.opt.common.Tools;
import net.wanji.opt.dao.mapper.evaluation.EventInfoMapper;
import net.wanji.opt.entity.evaluation.EventInfo;
import net.wanji.opt.servicev2.evaluation.EventInfoService;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;


/**
 * <p>
 * 交通事件信息
 * </p>
 *
 * @Author wangtao
 * @Date 2025-03-18
 */
@Slf4j
@Component
@Service
public class EventInfoServiceImpl extends BaseDubboInterfaceImpl<EventInfo> implements EventInfoService {

    @Resource
    private EventInfoMapper eventInfoMapper;


    @Override
    public BaseInterfaceMapper<EventInfo> getBaseInterfaceMapper() {
        return this.eventInfoMapper;
    }

    public List<Map<String, Object>> getListByStartAndEnd(Map<String, Object> map) {
        List<Map<String, Object>> list = this.eventInfoMapper.getListByStartAndEnd(map);
        for (Map<String, Object> data : list) {
            String dir = Tools.getMapValue("dir", data);
            if (Objects.nonNull(dir) && dir.length() > 2) {
                String dirName = CommonUtils.getEventHappenDirName(dir);
                data.put("dirName", dirName);
            }
        }
        return list;
    }

    @Override
    public void insertCrossSlowRunEventData(String startTime, String endTime) {
        long st = System.currentTimeMillis();

        eventInfoMapper.insertCrossSlowRunEventData(startTime, endTime);

        long et  = System.currentTimeMillis();

        log.info("路口缓行事件计算耗时：{}ms，时间：{} {}",et-st,startTime,endTime);
    }

}
