package net.wanji.opt.servicev2.implv2;

import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.TrunkLineMapper;
import net.wanji.opt.servicev2.TrunkLineService;
import net.wanji.opt.synthesis.pojo.CrossRealTimeAlarmEntity;
import net.wanji.opt.synthesis.pojo.TrunLineCrossStatusEntity;
import net.wanji.opt.synthesis.pojo.TrunkLineCrossProblemEntity;
import net.wanji.opt.synthesis.pojo.TrunkLineProblemDescribeEntity;
import net.wanji.opt.synthesis.pojo.vo.CrossOrGreenWaveTypeEntity;
import net.wanji.opt.vo2.TrunkLineCrossProblemVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
@Slf4j
public class TrunkLineImpl implements TrunkLineService {
    @Resource
    private TrunkLineMapper trunkLineMapper;

    @Override
    public List<TrunkLineProblemDescribeEntity> getTrunkLineProblemDescribe(String greenID , String time) {
        return trunkLineMapper.getTrunkLineProblemDescribe(greenID,time);
    }

    @Override
    public List<CrossRealTimeAlarmEntity> getTrunkLineRealTimeAlarm(Integer greenID, String time) {
        List<CrossRealTimeAlarmEntity> list = new ArrayList<>();
        CrossRealTimeAlarmEntity temp = trunkLineMapper.getTrunkLineRealTimeAlarm(greenID, time);
        CrossRealTimeAlarmEntity temp1 = new CrossRealTimeAlarmEntity();
        if(temp == null){
            temp1 = trunkLineMapper.getTrunkLineRealTimeAlarmPlan(greenID, time);
            if(temp1 != null){
                list.add(temp1);
            }
        }else{
            temp1 = trunkLineMapper.getTrunkLineRealTimeAlarmPlan1(greenID, time);
            list.add(temp);
            if(temp1 != null){
                list.add(temp1);
            }
        }
        return list;
    }

    @Override
    public List<TrunkLineCrossProblemEntity> getTrunkLineCrossProblem(Integer greenID, String time,Integer type) {
        //查询干线路口数据
        List<TrunkLineCrossProblemEntity> list = trunkLineMapper.getIODir(greenID);
        List<TrunkLineCrossProblemEntity> result1 = new ArrayList<>();
        List<TrunkLineCrossProblemEntity> result = new ArrayList<>();
        Integer inDir = 0;
        Integer outDir = 0;
        for(TrunkLineCrossProblemEntity temp : list){
            inDir = temp.getInDir();
            outDir = temp.getOutDir();
        }
        List<TrunLineCrossStatusEntity> crossStatusList = trunkLineMapper.getTrunkLineCrossStatus(greenID,type,time,inDir,outDir);
        for(TrunkLineCrossProblemEntity temp : list){
            for(TrunLineCrossStatusEntity temp1 : crossStatusList){
                if(temp.getSort().equals(1)){
                    if(temp.getCrossID().equals(temp1.getCrossID())){
                        if(temp1.getDirType().equals(temp.getOutDir()) ){
                            temp.setStatusR(temp1.getStatus());
                            temp.setDirectionR(temp.getOutDir().toString());
                            temp.setStartTime(temp1.getStartTime());
                        }
                    }
                }else if (temp.getSort().equals(list.size())){
                    if(temp.getCrossID().equals(temp1.getCrossID())){
                        if(temp1.getDirType().equals(temp.getInDir()) ){
                            temp.setStatusL(temp1.getStatus());
                            temp.setDirectionL(temp.getInDir().toString());
                            temp.setStartTime(temp1.getStartTime());
                        }
                    }
                }else{
                    if(temp.getCrossID().equals(temp1.getCrossID())){
                        if(temp1.getDirType().equals(temp.getOutDir()) ){
                            temp.setStatusR(temp1.getStatus());
                            temp.setDirectionR(temp.getOutDir().toString());
                        }
                        if(temp1.getDirType().equals(temp.getInDir()) ){
                            temp.setStatusL(temp1.getStatus());
                            temp.setDirectionL(temp.getInDir().toString());
                        }
                        temp.setStartTime(temp1.getStartTime());
                    }
                }
            }
            result1.add(temp);
        }
        for(int i = 0 ; i < result1.size() - 1 ; i ++){
            int j = i + 1;
            TrunkLineCrossProblemEntity temp = result1.get(i);
            TrunkLineCrossProblemEntity temp1 = result1.get(j);
            temp.setStatusL(temp1.getStatusL());
            temp.setDirectionL(temp1.getDirectionL());
            result.add(temp);
        }
        result.add(result1.get(result1.size() - 1));

        return result;
    }

    @Override
    public List<CrossOrGreenWaveTypeEntity> getGreenWaveTypeList(String greenID, String date) {
        return trunkLineMapper.getGreenWaveTypeList(greenID, date.replace("-", ""));
    }

    private Map<String, String> getDirName(Integer inDir, Integer outDir) {
        Map<String, String> result = new HashMap<>();

        if (inDir == 5 && outDir == 1) { // 南向北
            result.put("dir", "6");
            result.put("dirReverse", "0");
            result.put("dirName", "南向北");
            result.put("dirNameReverse", "北向南");
            return result;
        } else if (inDir == 7 && outDir == 3) { // 西向东
            result.put("dir", "4");
            result.put("dirReverse", "2");
            result.put("dirName", "西向东");
            result.put("dirNameReverse", "东向西");
            return result;
        } else if (inDir == 1 && outDir == 5) {
            result.put("dir", "0");
            result.put("dirReverse", "6");
            result.put("dirName", "北向南");
            result.put("dirNameReverse", "南向北");
        } else if (inDir == 3 && outDir == 7) {
            result.put("dir", "2");
            result.put("dirReverse", "4");
            result.put("dirName", "东向西");
            result.put("dirNameReverse", "西向东");
        }
        return result;
    }

    private String getStatusName(Integer status) {
        switch(status) {
            case -1:
            case 0:
                return "正常";
            case 1:
                return "失衡";
            case 2:
                return "拥堵";
            case 3:
                return "溢出";
            case 4:
                return "死锁";
            case 5:
                return "相位空放";
        }
        return "";
    }

    /**
     * @param inDir  进口方向
     * @param outDir 出口方向
     * @param dirs   方向数组
     * @param status 状态
     * @return 方向状态数组[方向1, 状态1, 方向2, 状态2]
     */
    private String[] getDirStatus(Integer inDir, Integer outDir, String[] dirs, Integer status) {
        Map<String, String> IODirMap = getDirName(inDir, outDir);
        String[] result = new String[4];

        if (Arrays.asList(dirs).contains(IODirMap.get("dir"))) {
            if (Objects.equals(IODirMap.get("dirName"), "南向北") || Objects.equals(IODirMap.get("dirName"), "西向东")) {
                result[0] = IODirMap.get("dirName");
                result[1] = getStatusName(status);
            } else {
                result[2] = IODirMap.get("dirName");
                result[3] = getStatusName(status);
            }

            if (Arrays.asList(dirs).contains(IODirMap.get("dirReverse"))) {
                if (Objects.equals(IODirMap.get("dirNameReverse"), "南向北") || Objects.equals(IODirMap.get("dirNameReverse"), "西向东")) {
                    result[0] = IODirMap.get("dirNameReverse");
                    result[1] = getStatusName(status);
                } else {
                    result[2] = IODirMap.get("dirNameReverse");
                    result[3] = getStatusName(status);
                }
            } else {
                if (Objects.equals(IODirMap.get("dirNameReverse"), "南向北") || Objects.equals(IODirMap.get("dirNameReverse"), "西向东")) {
                    result[0] = IODirMap.get("dirNameReverse");
                    result[1] = getStatusName(0);
                } else {
                    result[2] = IODirMap.get("dirNameReverse");
                    result[3] = getStatusName(0);
                }
            }
        } else {
            if (Objects.equals(IODirMap.get("dirName"), "南向北") || Objects.equals(IODirMap.get("dirName"), "西向东")) {
                result[0] = IODirMap.get("dirName");
                result[1] = getStatusName(0);
                result[2] = IODirMap.get("dirNameReverse");
                result[3] = getStatusName(0);
            } else {
                result[0] = IODirMap.get("dirNameReverse");
                result[1] = getStatusName(0);
                result[2] = IODirMap.get("dirName");
                result[3] = getStatusName(0);
            }
        }

        return result;
    }

}
