package net.wanji.opt.servicev2.judgeanalysis.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemAndStrategyDayMapper;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemAndStrategyDay;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemAndStrategyDayService;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 事件问题于策略关系实现类
 * </p>
 *
 * @author hwm
 * @since 2025-03-20
 */
@Service
@Slf4j
public class AnalysisProblemAndStrategyDayImpl extends ServiceImpl<AnalysisProblemAndStrategyDayMapper, AnalysisProblemAndStrategyDay> implements AnalysisProblemAndStrategyDayService {
    @Resource
    private AnalysisProblemAndStrategyDayMapper analysisProblemAndStrategyDayMapper;


    public void selectCountByTypeOfGreen(Integer dtStart,Integer dtEnd) {
        StopWatch stopWatch = new StopWatch("干线事件优化时长计算监控");
        stopWatch.start();
        //查询干线事件数据
        List<AnalysisProblemAndStrategyDay> greenEvenList = analysisProblemAndStrategyDayMapper.selectGreenEvent(dtStart,dtEnd);

        int size = greenEvenList.size();

        if (greenEvenList.size() > 0) {
            long st = System.currentTimeMillis();
            List<AnalysisProblemAndStrategyDay> saveList = new ArrayList<>();// greenEvenList.stream().filter(o->o.getGreenId().equals("1")).collect(Collectors.toList());
            long total = 0;
            for (AnalysisProblemAndStrategyDay vo : greenEvenList) {
                AnalysisProblemAndStrategyDay temp = analysisProblemAndStrategyDayMapper.getGreenStrategy(vo.getGreenId(), vo.getHappenStartTime());
                if (temp == null) {
                    vo.setOptStatus(0);
                } else {
                    vo.setOptStatus(1);
                    vo.setOptStartTime(temp.getOptStartTime());
                    vo.setOptEndTime(temp.getOptEndTime());
                    vo.setOptDuration(temp.getOptDuration());
                }
//                LocalDate previousDay = LocalDate.now().minusDays(1);
//                String formattedDate = previousDay.format(DateTimeFormatter.ofPattern("yyyyMMdd"));
                vo.setDt(vo.getDt());
                vo.setInsertTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                //干线数据设置路口ID为不存在的null字符串
                vo.setCrossId("null");
                saveList.add(vo);
                //写入数据
                if (saveList.size() % 100 == 0) {
                    analysisProblemAndStrategyDayMapper.insertProblemAndStrategy(saveList);
                    long et = System.currentTimeMillis();
                    total += 100;
                    log.info("干线事件优化记录挂接-批量插入优化挂接记录耗时：{}ms，已入库数量：{},总记录数：{}", et - st,total,size);
                    st = et;
                }
            }
            if (saveList.size() > 0) {
                total += saveList.size();
                analysisProblemAndStrategyDayMapper.insertProblemAndStrategy(saveList);
                long et = System.currentTimeMillis();
                log.info("干线事件优化记录挂接-批量插入优化挂接记录耗时：{}ms，已入库数量：{},总记录数：{}", et - st,total,size);
                saveList.clear();
            }
        }
        stopWatch.stop();
        log.info("干线事件优化时长计算监控 耗时：{}ms", stopWatch.getTotalTimeMillis());

    }

    public void selectCountByTypeOfCross(Integer dtStart,Integer dtEnd) {
        StopWatch stopWatch = new StopWatch("路口事件优化时长计算监控");
        stopWatch.start();

        //查询路口事件数据
        List<AnalysisProblemAndStrategyDay> crossEvenList = analysisProblemAndStrategyDayMapper.selectCrossEvent(dtStart,dtEnd);
        int size = crossEvenList.size();
        if (crossEvenList.size() > 0) {
            long st = System.currentTimeMillis();
            long total = 0;
            //循环查找路口事件数据是否有策略下发
            List<AnalysisProblemAndStrategyDay> saveList = new ArrayList<>();
            for (AnalysisProblemAndStrategyDay vo : crossEvenList) {
                AnalysisProblemAndStrategyDay temp = analysisProblemAndStrategyDayMapper.getCrossStrategy(vo.getCrossId(), vo.getHappenStartTime(), vo.getEventSerialNumber());
                if (temp == null) {
                    vo.setOptStatus(0);
                } else {
                    vo.setOptStatus(1);
                    vo.setOptStartTime(temp.getOptStartTime());
                    vo.setOptEndTime(temp.getOptEndTime());
                    vo.setOptDuration(temp.getOptDuration());
                }

//                LocalDate previousDay = LocalDate.now().minusDays(1);
//                String formattedDate = previousDay.format(DateTimeFormatter.ofPattern("yyyyMMdd"));
                vo.setDt(vo.getDt());
                vo.setInsertTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                //路口数据设置干线ID为不存在的ID
                vo.setGreenId("-1");
                saveList.add(vo);
                //写入数据--Error
                if (saveList.size() % 100 == 0) {
                    analysisProblemAndStrategyDayMapper.insertProblemAndStrategy(saveList);
                    long et = System.currentTimeMillis();
                    total += 100;
                    log.info("干线事件优化记录挂接-批量插入优化挂接记录耗时：{}ms，已入库数量：{},总记录数：{}", et - st,total,size);
                    st = et;
                }
            }
            if (saveList.size() > 0) {
                analysisProblemAndStrategyDayMapper.insertProblemAndStrategy(saveList);
                long et = System.currentTimeMillis();
                total += saveList.size();
                log.info("干线事件优化记录挂接-批量插入优化挂接记录耗时：{}ms，已入库数量：{},总记录数：{}", et - st,total,size);
                saveList.clear();
            }
        }

        stopWatch.stop();
        log.info("路口事件优化时长计算监控总耗时：{}ms", stopWatch.getTotalTimeMillis());

    }

}
