package net.wanji.opt.servicev2.judgeanalysis.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.swagger.models.auth.In;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.po.CrossTurnDataHistPO;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemAreaDayMapper;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemCrossDayMapper;
import net.wanji.opt.entity.GreenwaveHist;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemCrossDay;
import net.wanji.opt.entity.judgeanalysis.CrossPoint;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemAreaDayService;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemCrossDayService;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.locationtech.jts.geom.Geometry;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.stream.Collectors;

/**
* <p>
* 路口问题分析 服务实现类
* </p>
*
* @author fengyi
* @since 2025-03-13
*/
@Service
 public class AnalysisProblemAreaDayServiceImpl extends ServiceImpl<AnalysisProblemAreaDayMapper, AnalysisProblemCrossDay> implements AnalysisProblemAreaDayService {

	 @Resource
	 private AnalysisProblemAreaDayMapper analysisProblemAreaDayMapper;


	Map<String,Object> getParamMap(Integer areaId, Integer startTime, Integer endTime) {
		Map<String, Object> map = new HashMap<>();
		map.put("areaId", areaId);
		map.put("startTime", startTime);
		map.put("endTime", endTime);
		return map;
	}
	public List<AnalysisProblemCrossDay> selectCountByType(Integer areaId, Integer startTime, Integer endTime){

		return analysisProblemAreaDayMapper.selectCountByType(getParamMap(areaId,startTime,endTime));
	}

	@Override
	public Map<String,Object> selectAreaProblemTrend(Integer areaId, Integer startTime, Integer endTime) throws Exception {

		List<AnalysisProblemCrossDay> list = analysisProblemAreaDayMapper.selectAreaProblemTrend(getParamMap(areaId,startTime,endTime));

		Date sd = DateUtil.parse(startTime.toString(),EsDateIndexUtil.YMD_FORMATTER);
		Date ed = DateUtil.parse(endTime.toString(),EsDateIndexUtil.YMD_FORMATTER);
		//存放时段
		Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis("4", DateUtil.formatDate(sd,EsDateIndexUtil.YMD_HM_FORMATTER), DateUtil.formatDate(ed,EsDateIndexUtil.YMD_HM_FORMATTER));
		//======================================================================================================//
		//存放所有数据
		List<Map<String, Object>> allList = new ArrayList<>();
		//按事件类型分组
		Map<String, List<AnalysisProblemCrossDay>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getEventType(), TreeMap::new, Collectors.toList()));

		for (Map.Entry<String, List<AnalysisProblemCrossDay>> entry : groupByDir.entrySet()) {
			String key = entry.getKey();
			//同一类事件每个日期问题数量
			List<AnalysisProblemCrossDay> value = entry.getValue();
			//事件名称
			String typeName = value.get(0).getTypeName();

			//有实际数据的时刻点
			Set<String> timeList = value.stream().map(po -> po.getDt().toString()).collect(Collectors.toSet());
			//补充缺少时段数据，保留时段字段默认值
			for (String timeSec : sortedSet) {
				if (!timeList.contains(timeSec)) {
					AnalysisProblemCrossDay tmp = new AnalysisProblemCrossDay();
					tmp.setDt(Integer.valueOf(timeSec));
					value.add(tmp);
				}
			}
			//补充缺失数据后按时间升序排序
			value = value.stream().sorted(Comparator.comparing(AnalysisProblemCrossDay::getDt)).collect(Collectors.toList());
			//提取数量属性
			List<Integer> numberList = value.stream().map(AnalysisProblemCrossDay::getEventNumber).collect(Collectors.toList());

			List<Integer> durationList = value.stream().map(AnalysisProblemCrossDay::getEventTotalTime).collect(Collectors.toList());

			Map<String, Object> mapList = new HashMap<>();
			mapList.put("eventType", key);
			mapList.put("eventLabel", typeName);
			mapList.put("list", numberList);
			mapList.put("durationList", durationList);
			allList.add(mapList);

		}

		JSONObject jsonObject = new JSONObject();
		jsonObject.put("timeList", sortedSet);
		jsonObject.put("dataList", allList);

		return jsonObject;
	}

	@Override
	public List<Map<String, Object>> selectCrossAllProblemNums(String eventType,Integer areaId, Integer startTime, Integer endTime) throws Exception {
		Map<String,Object> param = getParamMap(areaId,startTime,endTime);
		param.put("eventType",eventType);
		return analysisProblemAreaDayMapper.selectCrossAllProblemNums(param);
	}


	@Override
	public List<Map<String, Object>> selectGreenAllProblemNums(String eventType,Integer areaId, Integer startTime, Integer endTime) throws Exception {
		Map<String,Object> param = getParamMap(areaId,startTime,endTime);
		param.put("eventType",eventType);
		return analysisProblemAreaDayMapper.selectGreenAllProblemNums(param);
	}


}
