package net.wanji.opt.servicev2.judgeanalysis.impl;

import com.alibaba.fastjson.JSONObject;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemCrossDayMapper;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemGreenDay;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemGreenDayMapper;
import net.wanji.opt.entity.judgeanalysis.CrossPoint;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemGreenDayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import net.wanji.opt.synthesis.pojo.CrossOptAnalysisEntity;
import net.wanji.opt.vo2.CrossOptAnalysisVO;
import org.locationtech.jts.geom.Geometry;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import javax.annotation.Resource;
import java.text.DateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
* <p>
* 绿波问题分析 服务实现类
* </p>
*
* @author fengyi
* @since 2025-03-13
*/
@Service
 public class AnalysisProblemGreenDayServiceImpl extends ServiceImpl<AnalysisProblemGreenDayMapper, AnalysisProblemGreenDay> implements AnalysisProblemGreenDayService {

	 @Resource
	 private AnalysisProblemGreenDayMapper analysisProblemGreenDayMapper;

	@Resource
	private AnalysisProblemCrossDayMapper analysisProblemCrossDayMapper;

	
	 /**
	 * 绿波问题分析详情
	 * @param id
	 * @return
	 */
	 @Override
	 public AnalysisProblemGreenDay info(Long id) {
	
	 	return getById(id);
	 }
	
	 /**
	 * 绿波问题分析新增
	 * @param param 根据需要进行传值
	 * @return
	 */
	 @Override
	 public void add(AnalysisProblemGreenDay param) {
	
	 	save(param);
	 }
	
	 /**
	 * 绿波问题分析修改
	 * @param param 根据需要进行传值
	 * @return
	 */
	 @Override
	 public void modify(AnalysisProblemGreenDay param) {
	
	 	updateById(param);
	 }
	
	 /**
	 * 绿波问题分析删除(单个条目)
	 * @param id
	 * @return
	 */
	 @Override
	 public void remove(Long id) {
	 	removeById(id);
	 }
	
	 /**
	 * 绿波问题分析删除(多个条目)
	 * @param ids
	 * @return
	 */
	 @Override
	 public void removes(List<Long> ids) {
	
	 	removeByIds(ids);
	 }
	/**
	 * 路口区域id数据添加
	 * @return
	 */
	@Override
	public void updateGreenAreaIdByCondition() {
		//判断路口的areaId是否存在
		List<CrossPoint> greenPoints = analysisProblemCrossDayMapper.findGreenPoint();
		if(greenPoints.size()>0)
		{
			//获取区域id的坐标范围
			List<CrossPoint> areaPont = analysisProblemCrossDayMapper.findAreaPoint();
			//循环判断是否存在区域内
			for(CrossPoint greenPoint:greenPoints){
				int i = 0;
				for(CrossPoint areaPoint:areaPont){
					Geometry geo = net.wanji.common.gts.GeomsConvertUtil.genGeometry(greenPoint.getPoint());
					Geometry geo1 = net.wanji.common.gts.GeomsConvertUtil.genGeometry(areaPoint.getPoint());
					if(geo.within(geo1)){
						//如果匹配则更新id
						analysisProblemCrossDayMapper.updateGreenAreaIdByid(areaPoint.getId(),greenPoint.getId());
						i = 1;
					}
				}
				if(i==0){
					//未匹配则把区域id设置为0
					analysisProblemCrossDayMapper.updateGreenAreaIdByid("0",greenPoint.getId());
				}
			}
		}

	}

	@Override
	public List<AnalysisProblemGreenDay> getListGreenByProblemSum(Integer greenId,Integer startTime,Integer endTime) {
		Map<String,Object> map=new HashMap<>();
		map.put("greenId",greenId);
		map.put("startTime",startTime);
		map.put("endTime",endTime);
		return  analysisProblemGreenDayMapper.getListGreenByProblemSum(map);

	}

	@Override
	public Map<String, Object> getListGreenByProblemTrend(Integer greenId, Integer startTime, Integer endTime) throws Exception {
		Map<String,Object> map=new HashMap<>();
		map.put("greenId",greenId);
		map.put("startTime",startTime);
		map.put("endTime",endTime);

	    List<AnalysisProblemGreenDay> list=	analysisProblemGreenDayMapper.getListGreenByProblemTrend(map);
	    Date sd= DateUtil.parse(startTime.toString(),EsDateIndexUtil.YMD_FORMATTER);
		Date ed=DateUtil.parse(endTime.toString(),EsDateIndexUtil.YMD_FORMATTER);
		//存放全部时段
		Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis("4", DateUtil.formatDate(sd,EsDateIndexUtil.YMD_HM_FORMATTER), DateUtil.formatDate(ed,EsDateIndexUtil.YMD_HM_FORMATTER));
	    List<Map<String,Object>>allList=new ArrayList<>();

		Map<String, List<AnalysisProblemGreenDay>> groupByType = list.stream().collect(Collectors.groupingBy(o -> o.getEventType(), TreeMap::new, Collectors.toList()));
		for (Map.Entry<String,List<AnalysisProblemGreenDay>>entry:groupByType.entrySet()){
			String key = entry.getKey();
			List<AnalysisProblemGreenDay> value = entry.getValue();
			String typeName = value.get(0).getTypeName();

			// 构造一个Map用于快速查找已有数据点
			Map<String, Integer> existingDataPoints = value.stream()
					.collect(Collectors.toMap(po -> po.getDt().toString(), AnalysisProblemGreenDay::getEventNumber));
			// 构造一个Map用于快速查找时长已有数据点
			Map<String, Integer> existingDate = value.stream()
					.collect(Collectors.toMap(po -> po.getDt().toString(), AnalysisProblemGreenDay::getEventTotalTime));

			// 对于每个时间点，如果不存在数据则补0
			List<Integer> numberList = sortedSet.stream()
					.map(timePoint -> existingDataPoints.getOrDefault(timePoint, 0))
					.collect(Collectors.toList());
			// 对于每个时间点，如果时长不存在数据则补0
			List<Integer> durationList = sortedSet.stream()
					.map(timePoint -> existingDate.getOrDefault(timePoint, 0))
					.collect(Collectors.toList());
			Map<String,Object>mapList=new HashMap<>();
			 mapList.put("eventType",key);
			 mapList.put("eventLabel",typeName);
			 mapList.put("list",numberList);
			 mapList.put("durationList",durationList);
			 allList.add(mapList);

		}

		JSONObject jsonObject = new JSONObject();
		jsonObject.put("timeList",new ArrayList<>(sortedSet));
		jsonObject.put("dataList",allList);
		return jsonObject;
	}

	@Override
	public CrossOptAnalysisVO getGreenOptAnalysis(String greenId, String startTime, String endTime) {
		List<CrossOptAnalysisEntity> entityList = analysisProblemCrossDayMapper.getCrossOptAnalysis(greenId,null, startTime,endTime);
		CrossOptAnalysisVO crossOptAnalysisVO = new CrossOptAnalysisVO();

		for (CrossOptAnalysisEntity c : entityList) {
			if (c.getStatus().equals("已优化")) {
				crossOptAnalysisVO.setOptimized(c.getNum());
			} else if (c.getStatus().equals("未优化")) {
				crossOptAnalysisVO.setUnoptimized(c.getNum());
			}
		}
		crossOptAnalysisVO.setTotal(crossOptAnalysisVO.getOptimized() + crossOptAnalysisVO.getUnoptimized());

		return crossOptAnalysisVO;
	}

}
