package net.wanji.opt.servicev2.judgeanalysis.impl;
import com.alibaba.fastjson.JSONObject;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemGreenDay;
import net.wanji.opt.entity.judgeanalysis.AnalysisProblemGreenHour;
import net.wanji.opt.dao.mapper.judgeanalysis.AnalysisProblemGreenHourMapper;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisProblemGreenHourService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import javax.annotation.Resource;
import java.text.ParseException;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
* <p>
* 干线按小时粒度问题分析 服务实现类
* </p>
*
* @author fengyi
* @since 2025-03-21
*/
@Service
 public class AnalysisProblemGreenHourServiceImpl extends ServiceImpl<AnalysisProblemGreenHourMapper, AnalysisProblemGreenHour> implements AnalysisProblemGreenHourService {

	 @Resource
	 private AnalysisProblemGreenHourMapper analysisProblemGreenHourMapper;

	
	 /**
	 * 干线按小时粒度问题分析详情
	 * @param id
	 * @return
	 */
	 @Override
	 public AnalysisProblemGreenHour info(Long id) {
	
	 	return getById(id);
	 }
	
	 /**
	 * 干线按小时粒度问题分析新增
	 * @param param 根据需要进行传值
	 * @return
	 */
	 @Override
	 public void add(AnalysisProblemGreenHour param) {
	
	 	save(param);
	 }
	
	 /**
	 * 干线按小时粒度问题分析修改
	 * @param param 根据需要进行传值
	 * @return
	 */
	 @Override
	 public void modify(AnalysisProblemGreenHour param) {
	
	 	updateById(param);
	 }
	
	 /**
	 * 干线按小时粒度问题分析删除(单个条目)
	 * @param id
	 * @return
	 */
	 @Override
	 public void remove(Long id) {
	 	removeById(id);
	 }
	
	 /**
	 * 干线按小时粒度问题分析删除(多个条目)
	 * @param ids
	 * @return
	 */
	 @Override
	 public void removes(List<Long> ids) {
	
	 	removeByIds(ids);
	 }

	@Override
	public Map<String, Object> directionGreenByProblemTrend(Integer greenId, String eventType, Integer startTime, Integer endTime) throws ParseException {
		Map<String,Object> map=new HashMap<>();
		map.put("greenId",greenId);
		map.put("eventType",eventType);
		map.put("startTime",startTime);
		map.put("endTime",endTime);
		List<AnalysisProblemGreenHour> list=analysisProblemGreenHourMapper.directionGreenByProblemTrend(map);
		Date sd=DateUtil.parse(startTime.toString(), EsDateIndexUtil.YMD_FORMATTER);
		Date ed= DateUtil.parse(endTime.toString(),EsDateIndexUtil.YMD_FORMATTER);
		//存放期间全部时段
		Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis("4", DateUtil.formatDate(sd,EsDateIndexUtil.YMD_HM_FORMATTER), DateUtil.formatDate(ed,EsDateIndexUtil.YMD_HM_FORMATTER));

		List<Map<String,Object>> allList=new ArrayList<>();
		//按照路口分组
		Map<String, List<AnalysisProblemGreenHour>> collect = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId(), TreeMap::new, Collectors.toList()));
		for (Map.Entry<String,List<AnalysisProblemGreenHour>>entry:collect.entrySet()){

			String key = entry.getKey();
			//每个路口的信息
			List<AnalysisProblemGreenHour> value = entry.getValue();

			String crossName = value.get(0).getCrossName();

			String eventTypeEvery = value.get(0).getEventType();

			String typeName = value.get(0).getTypeName();
			//构造一个Map用于快速查找已有数据点
			Map<String, Integer> existingDataPoints = value.stream()
					.collect(Collectors.toMap(po -> po.getDt().toString(), AnalysisProblemGreenHour::getEventNumber));

			List<Integer> numberList = sortedSet.stream().map(timepoint -> existingDataPoints.getOrDefault(timepoint, 0))
					.collect(Collectors.toList());

			Map<String,Object>mapList=new HashMap<>();
			mapList.put("crossId",key);
			mapList.put("crossName",crossName);
			mapList.put("eventType",eventTypeEvery);
			mapList.put("eventLabel",typeName);
			mapList.put("list",numberList);
			allList.add(mapList);
		}

		JSONObject jsonObject=new JSONObject();
		jsonObject.put("dataList",allList);
		jsonObject.put("timeList",sortedSet);
		return jsonObject;
	}

	@Override
	public Map<String, Object> detailGreenByProblemTrend(Integer greenId, String eventType, Integer startTime, Integer endTime) throws Exception {
		Map<String,Object> map=new HashMap<>();
		map.put("greenId",greenId);
		map.put("eventType",eventType);
		map.put("startTime",startTime);
		map.put("endTime",endTime);
		//按照小时
		List<AnalysisProblemGreenHour>listTime=analysisProblemGreenHourMapper.detailGreenByProblemTrend(map);

		//按照路口
		List<AnalysisProblemGreenHour>listCross=analysisProblemGreenHourMapper.detailGreenByCrossProblem(map);

		//查询路段
		List<AnalysisProblemGreenHour> listCrossLine = analysisProblemGreenHourMapper.detailGreenByRoadLineProblem(map);
		//所有时间段
		Set<String> allHourset=new LinkedHashSet();
		//所有路口
		Set<String> allCorss=new LinkedHashSet();
		//所有路段
		List<String> listRoadLine=new ArrayList<>();
		String eventTypeCross="";
		String typeNameCross="";

		//定时时间格式
		DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("HH:mm");
        //取出时间
  		for (AnalysisProblemGreenHour singtime:listTime){
			String stringHour = singtime.getHours_window_start_time();
			 eventTypeCross = singtime.getEventType();
			 typeNameCross = singtime.getTypeName();
			if (stringHour!=null && !"".equals(stringHour)){
				LocalTime startTimeHour = LocalTime.parse(stringHour, dateTimeFormatter);
                 // 对时间加一个小时得到结束时间
				LocalTime endTimeHours = startTimeHour.plusHours(1);
				// 格式化开始时间和结束时间为所需的字符串格式
				String formattedTimeRange=startTimeHour.format(dateTimeFormatter)+"-"+endTimeHours.format(dateTimeFormatter);
				allHourset.add(formattedTimeRange);
			}
		  }
		 //取出路口
		for (AnalysisProblemGreenHour singcorss:listCross){
			eventTypeCross = singcorss.getEventType();
			typeNameCross = singcorss.getTypeName();
			String crossName = singcorss.getCrossName();
			allCorss.add(crossName);
		}
		//取出路段
		for (AnalysisProblemGreenHour roadLine: listCrossLine){
			String roadName = roadLine.getRoadName();
			listRoadLine.add(roadName);
		}
		JSONObject jsonObject =new JSONObject();
		jsonObject.put("eventLabel",typeNameCross);
		jsonObject.put("eventType",eventTypeCross);
		jsonObject.put("mainCongestCrossName",allCorss);
		jsonObject.put("mainCongestRoadName",listRoadLine);
		jsonObject.put("timeList",allHourset);
		return jsonObject;
	}
}
