package net.wanji.opt.servicev2.judgeanalysis.impl;

import com.alibaba.fastjson.JSONObject;
import net.wanji.opt.dao.mapper.judgeanalysis.CrossSignalPlanMapper;
import net.wanji.opt.servicev2.judgeanalysis.CrossSignalPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class CrossSignalPlanServiceImpl implements CrossSignalPlanService {
    @Autowired
    private CrossSignalPlanMapper crossSignalPlanMapper;
    @Override
    public JSONObject getByDateScope(String crossId, String startTime, String endTime) {

        List<String> dateRange = generateDateRange(startTime, endTime);
        //查询路口特殊日期
        List<Map<String,String>> specialDates = crossSignalPlanMapper.getSpecialDate(crossId,startTime,endTime);
        List<Map<String,String>> result = new ArrayList<>();
        dateRange.forEach(x->{
            //非特殊日期周期查询
            if(ObjectUtils.isEmpty(specialDates)){
                int weekNumber = getDayOfWeekNumber(x);
                List<Map<String,String>> scopeList = crossSignalPlanMapper.getByDateScope(crossId,weekNumber);
                scopeList.forEach(t->{
                    t.put("startTime",x+" "+t.get("startTime")+":00");
                    t.put("endTime",x+" "+t.get("endTime")+":00");
                });
                result.addAll(scopeList);
            }else {
                boolean specialBool = false;
                for (Map<String, String> specialDate : specialDates) {
                    DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                    DateTimeFormatter formatter1 = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                    String special_date = specialDate.get("special_date");
                    LocalDateTime specialTime = LocalDateTime.parse(special_date,formatter);
                    //特殊日期周期查询
                    if(x.equals(specialTime.format(formatter1))){
                        String planId = String.valueOf(specialDate.get("plan_id"));
                        List<Map<String,String>> scopeList = crossSignalPlanMapper.getSpecialByDateScope(crossId,Integer.valueOf(planId));
                        scopeList.forEach(t->{
                            t.put("startTime",x+" "+t.get("startTime")+":00");
                            t.put("endTime",x+" "+t.get("endTime")+":00");
                        });
                        result.addAll(scopeList);
                        specialBool = true;
                        break;
                    }
                }
                //非特殊日期周期查询
                if(!specialBool){
                    int weekNumber = getDayOfWeekNumber(x);
                    List<Map<String,String>> scopeList = crossSignalPlanMapper.getByDateScope(crossId,weekNumber);
                    scopeList.forEach(t->{
                        t.put("startTime",x+" "+t.get("startTime")+":00");
                        t.put("endTime",x+" "+t.get("endTime")+":00");
                    });
                    result.addAll(scopeList);
                }
            }
        });

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("list",result);
        return jsonObject;
    }

    /**
     * 获取开始时间和结束时间之间的所有日期，格式为 yyyy-MM-dd HH:mm:ss
     *
     * @param startDate 开始时间（格式：yyyy-MM-dd HH:mm:ss）
     * @param endDate   结束时间（格式：yyyy-MM-dd HH:mm:ss）
     * @return 包含所有日期的 List<String>
     */
    public static List<String> generateDateRange(String startDate, String endDate) {
        List<String> dateList = new ArrayList<>();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        DateTimeFormatter formatter1 = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        // 将字符串解析为 LocalDate
        LocalDateTime start = LocalDateTime.parse(startDate,formatter);
        LocalDateTime end = LocalDateTime.parse(endDate,formatter);

        // 遍历日期范围
        while (!start.isAfter(end)) {
            dateList.add(start.format(formatter1));
            start = start.plusDays(1); // 增加一天
        }

        return dateList;
    }

    /**
     * 获取指定日期是周几的数字表示（周一为 1，周日为 7）
     *
     * @param date 输入日期，格式为 yyyy-MM-dd
     * @return 周几的数字（1 表示周一，7 表示周日）
     */
    public static int getDayOfWeekNumber(String date) {
        // 定义日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        // 将字符串解析为 LocalDate
        LocalDate localDate = LocalDate.parse(date, formatter);

        // 获取周几
        DayOfWeek dayOfWeek = localDate.getDayOfWeek();

        // 返回数字表示（周一为 1，周日为 7）
        return dayOfWeek.getValue(); // getValue() 返回 1（周一）到 7（周日）
    }
}
