package net.wanji.opt.servicev2.report.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.opt.common.TimeIntervalMerger;
import net.wanji.opt.common.enums.WeekEnum;
import net.wanji.opt.dao.mapper.report.AnalysisGreenWaveOptimizeWeekMapper;
import net.wanji.opt.entity.report.AnalysisGreenWaveOptimizeWeek;
import net.wanji.opt.servicev2.report.AnalysisGreenWaveOptimizeWeekService;
import net.wanji.opt.vo2.report.AnalysisGreenWaveOptimizeWeekVO;
import net.wanji.opt.vo2.report.GreenWaveOptimizeAnalysisDataDTO;
import net.wanji.opt.vo2.report.GreenWaveOptimizeAnalysisDataVO;
import net.wanji.opt.vo2.report.GreenWaveOptimizeAnalysisResultResponseDTO;
import net.wanji.opt.vo2.report.ImportGreenReportAnalysisDataVO;
import net.wanji.opt.vo2.report.ImportGreenReportAnalysisResponseVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author fengyi
 * @since 2025-04-09
 */
@Service
public class AnalysisGreenWaveOptimizeWeekServiceImpl extends ServiceImpl<AnalysisGreenWaveOptimizeWeekMapper, AnalysisGreenWaveOptimizeWeek> implements AnalysisGreenWaveOptimizeWeekService {

    @Resource
    private AnalysisGreenWaveOptimizeWeekMapper analysisGreenWaveOptimizeWeekMapper;
    @Autowired
    private GreenwaveInfoMapper greenwaveInfoMapper;

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @Override
    public AnalysisGreenWaveOptimizeWeek info(Long id) {

        return getById(id);
    }

    /**
     * 新增
     *
     * @param param 根据需要进行传值
     * @return
     */
    @Override
    public void add(AnalysisGreenWaveOptimizeWeek param) {

        save(param);
    }

    /**
     * 修改
     *
     * @param param 根据需要进行传值
     * @return
     */
    @Override
    public void modify(AnalysisGreenWaveOptimizeWeek param) {

        updateById(param);
    }

    /**
     * 删除(单个条目)
     *
     * @param id
     * @return
     */
    @Override
    public void remove(Long id) {
        removeById(id);
    }

    /**
     * 删除(多个条目)
     *
     * @param ids
     * @return
     */
    @Override
    public void removes(List<Long> ids) {

        removeByIds(ids);
    }

    private SimpleDateFormat format = new SimpleDateFormat("HH:mm");

    @Override
    public GreenWaveOptimizeAnalysisResultResponseDTO getTotalityData(Integer year, Integer week, String ids) {
        Integer yearWeek = Integer.valueOf(year + "" + week);
        Integer yearWeekLast = Integer.valueOf(year + "" + (week - 1));
        List<Integer> greenIdList = new ArrayList<>();
        if (ObjectUtils.isEmpty(ids)) {
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();
            greenIdList = greenwaveInfoPOList.stream().map(GreenwaveInfoPO::getId).collect(Collectors.toList());
        } else {
            greenIdList = Arrays.stream(ids.split(","))
                    .map(String::trim) // 去除可能的空格
                    .filter(s -> !s.isEmpty()) // 过滤掉空字符串
                    .map(Integer::parseInt) // 转换为整数
                    .collect(Collectors.toList());
        }
        GreenWaveOptimizeAnalysisDataDTO greenWaveOptimizeAnalysisDataDTO = analysisGreenWaveOptimizeWeekMapper.getTotalityData(yearWeek, greenIdList);

        GreenWaveOptimizeAnalysisResultResponseDTO optimizeAnalysisResultResponseDTO = new GreenWaveOptimizeAnalysisResultResponseDTO();
        List<GreenWaveOptimizeAnalysisDataVO> dataVOList = new ArrayList<>();
        List<String> timeString = new ArrayList<>();
        if (ObjectUtils.isEmpty(greenWaveOptimizeAnalysisDataDTO)) {
            optimizeAnalysisResultResponseDTO.setWeekOptimizeCount("0次");
            optimizeAnalysisResultResponseDTO.setWeekOptimizeDuration("0小时");
            optimizeAnalysisResultResponseDTO.setWeekCapacitySituation("无变化");
            optimizeAnalysisResultResponseDTO.setWeekStopTimesSituation("无变化");
            optimizeAnalysisResultResponseDTO.setWeekTravelTimeSituation("无变化");
            optimizeAnalysisResultResponseDTO.setWeekDelayTimeSituation("无变化");
            optimizeAnalysisResultResponseDTO.setWeekAvgSpeedSituation("无变化");
            optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("无变化");
        } else {
            optimizeAnalysisResultResponseDTO.setWeekOptimizeCount(String.valueOf(greenWaveOptimizeAnalysisDataDTO.getOptimizeCount()) + "次");
            optimizeAnalysisResultResponseDTO.setWeekOptimizeDuration(String.valueOf(greenWaveOptimizeAnalysisDataDTO.getOptimizeDuration() / 60 / 60) + "小时");
            GreenWaveOptimizeAnalysisDataDTO analysisDataDTO = greenWaveOptimizeAnalysisDataDTO;
            GreenWaveOptimizeAnalysisDataVO analysisDataVO = new GreenWaveOptimizeAnalysisDataVO();
            analysisDataVO.setCapacity(analysisDataDTO.getCapacity());
            analysisDataVO.setStopTimes(analysisDataDTO.getStopTimes());
            analysisDataVO.setDelayTime(analysisDataDTO.getDelayTime());
            analysisDataVO.setTravelTime(analysisDataDTO.getTravelTime());
            analysisDataVO.setAvgSpeed(analysisDataDTO.getAvgSpeed());
            dataVOList.add(getAnalysisDataLastVO(analysisDataVO));
            timeString.add("本周");
            timeString.add("上周");
            GreenWaveOptimizeAnalysisDataVO analysisDataLastVO = new GreenWaveOptimizeAnalysisDataVO();
            analysisDataLastVO.setCapacity(analysisDataDTO.getLastCapacity());
            analysisDataLastVO.setStopTimes(analysisDataDTO.getLastStopTimes());
            analysisDataLastVO.setDelayTime(analysisDataDTO.getLastDelayTime());
            analysisDataLastVO.setTravelTime(analysisDataDTO.getLastTravelTime());
            analysisDataLastVO.setAvgSpeed(analysisDataDTO.getLastAvgSpeed());
            dataVOList.add(getAnalysisDataLastVO(analysisDataLastVO));
            int weekCapacitySituation = 0;
            long weekStopTimesSituation = 0;
            int weekTravelTimeSituation = 0;
            long weekDelayTimeSituation = 0;
            long weekAvgSpeedSituation = 0;
            //本周通行能力同比情况
            if (analysisDataDTO.getLastCapacity() != 0) {
                weekCapacitySituation = (analysisDataDTO.getCapacity() - analysisDataDTO.getLastCapacity()) * 100 / analysisDataDTO.getLastCapacity();
                if (weekCapacitySituation > 0) {
                    optimizeAnalysisResultResponseDTO.setWeekCapacitySituation("提升" + weekCapacitySituation + "%");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekCapacitySituation("降低" + Math.abs(weekCapacitySituation) + "%");
                }
            } else {
                optimizeAnalysisResultResponseDTO.setWeekCapacitySituation("提升100%");
            }
            //本周通停车次数同比情况
            if (analysisDataDTO.getLastStopTimes() != 0) {
                weekStopTimesSituation = Math.round((analysisDataDTO.getStopTimes() - analysisDataDTO.getLastStopTimes()) * 100 / analysisDataDTO.getLastStopTimes());
                if (weekStopTimesSituation > 0) {
                    optimizeAnalysisResultResponseDTO.setWeekStopTimesSituation("增加" + weekStopTimesSituation + "%");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekStopTimesSituation("减少" + Math.abs(weekStopTimesSituation) + "%");
                }
            }
            //本周通行程时间数同比情况
            if (analysisDataDTO.getLastTravelTime() != 0) {
                weekTravelTimeSituation = (analysisDataDTO.getTravelTime() - analysisDataDTO.getLastTravelTime()) * 100 / analysisDataDTO.getLastTravelTime();
                if (weekTravelTimeSituation > 0) {
                    optimizeAnalysisResultResponseDTO.setWeekTravelTimeSituation("增加" + weekTravelTimeSituation + "%");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekTravelTimeSituation("减少" + Math.abs(weekTravelTimeSituation) + "%");
                }
            }
            //本周通延误时间同比情况
            if (analysisDataDTO.getLastDelayTime() != 0) {
                weekDelayTimeSituation = Math.round((analysisDataDTO.getDelayTime() - analysisDataDTO.getLastDelayTime()) * 100 / analysisDataDTO.getLastDelayTime());
                if (weekDelayTimeSituation > 0) {
                    optimizeAnalysisResultResponseDTO.setWeekDelayTimeSituation("增加" + weekDelayTimeSituation + "%");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekDelayTimeSituation("减少" + Math.abs(weekDelayTimeSituation) + "%");
                }
            }
            //本周通平均速度同比情况
            if (analysisDataDTO.getLastAvgSpeed() != 0) {
                weekAvgSpeedSituation = Math.round((analysisDataDTO.getAvgSpeed() - analysisDataDTO.getLastAvgSpeed()) * 100 / analysisDataDTO.getLastAvgSpeed());
                if (weekAvgSpeedSituation > 0) {
                    optimizeAnalysisResultResponseDTO.setWeekAvgSpeedSituation("提升" + weekAvgSpeedSituation + "%");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekAvgSpeedSituation("降低" + Math.abs(weekAvgSpeedSituation) + "%");
                }
            }
            //本周总体情况
            if (weekCapacitySituation >= 0 && weekTravelTimeSituation <= 0) {
                if (weekCapacitySituation >= 5) {
                    optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("明显提升");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("略有提升");
                }
            } else if (weekCapacitySituation >= 0 && weekTravelTimeSituation > 0) {
                if (weekCapacitySituation - weekTravelTimeSituation > 0) {
                    optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("略有提升");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("略有下降");
                }
            } else if (weekCapacitySituation < 0 && weekTravelTimeSituation <= 0) {
                if (weekCapacitySituation - weekTravelTimeSituation > 0) {
                    optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("略有提升");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("略有下降");
                }
            } else if (weekCapacitySituation < 0 && weekTravelTimeSituation > 0) {
                if (Math.abs(weekCapacitySituation) >= 5) {
                    optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("明显下降");
                } else {
                    optimizeAnalysisResultResponseDTO.setWeekTotalitySituation("略有下降");
                }
            }
        }
        optimizeAnalysisResultResponseDTO.setTimeList(timeString);
        optimizeAnalysisResultResponseDTO.setDataList(dataVOList);

        return optimizeAnalysisResultResponseDTO;
    }

    private GreenWaveOptimizeAnalysisDataVO getAnalysisDataLastVO(GreenWaveOptimizeAnalysisDataVO analysisDataLastVO) {
        analysisDataLastVO.setAvgSpeed(Math.round(analysisDataLastVO.getAvgSpeed() * 100) / 100.0);
        analysisDataLastVO.setStopTimes(Math.round(analysisDataLastVO.getStopTimes() * 100) / 100.0);
        analysisDataLastVO.setDelayTime(Math.round(analysisDataLastVO.getDelayTime() * 100) / 100.0);
        return analysisDataLastVO;
    }

    @Override
    public List<ImportGreenReportAnalysisResponseVO> getProblemDetail(Integer year, Integer week, String ids) {
        Integer yearWeek = Integer.valueOf(year + "" + week);
        Integer yearWeekLast = Integer.valueOf(year + "" + (week - 1));
        List<Integer> greenIdList = new ArrayList<>();
        if (ObjectUtils.isEmpty(ids)) {
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();
            greenIdList = greenwaveInfoPOList.stream().map(GreenwaveInfoPO::getId).collect(Collectors.toList());
        } else {
            greenIdList = Arrays.stream(ids.split(","))
                    .map(String::trim) // 去除可能的空格
                    .filter(s -> !s.isEmpty()) // 过滤掉空字符串
                    .map(Integer::parseInt) // 转换为整数
                    .collect(Collectors.toList());
        }

        List<AnalysisGreenWaveOptimizeWeekVO> optimizeWeeks = analysisGreenWaveOptimizeWeekMapper.selectListByYearWeek(yearWeek);
        Map<Integer, List<AnalysisGreenWaveOptimizeWeekVO>> greenMap = optimizeWeeks.stream().collect(Collectors.groupingBy(AnalysisGreenWaveOptimizeWeekVO::getGreenId));
        List<ImportGreenReportAnalysisResponseVO> analysisResponseVOS = new ArrayList<>();
        for (Integer greenId : greenIdList) {
            Integer allCapacity = 0;
            Integer allCapacityLast = 0;
            Double allCongestIndex = 0.0;
            Double allCongestIndexLast = 0.0;
            Double allSpeed = 0.0;
            Double allSpeedLast = 0.0;
            Integer allTravelTime = 0;
            Integer allTravelTimeLast = 0;
            ImportGreenReportAnalysisResponseVO analysisResponseVO = new ImportGreenReportAnalysisResponseVO();
            List<ImportGreenReportAnalysisDataVO> analysisDataVOS = new ArrayList<>();
            List<AnalysisGreenWaveOptimizeWeekVO> waveOptimizeWeeks = greenMap.get(greenId);
            GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenId);
            analysisResponseVO.setGreenId(String.valueOf(greenId));
            analysisResponseVO.setGreenName(greenwaveInfoPO.getName());
            if (ObjectUtils.isEmpty(waveOptimizeWeeks)) {
//                analysisResponseVOS.add(analysisResponseVO);
                continue;
            }
            List<String> timeList = new ArrayList<>();
            for (AnalysisGreenWaveOptimizeWeekVO optimizeWeek : waveOptimizeWeeks) {
                ImportGreenReportAnalysisDataVO analysisDataVO = new ImportGreenReportAnalysisDataVO();
                Integer capacity = ObjectUtils.isEmpty(optimizeWeek.getCapacity()) ? 0 : optimizeWeek.getCapacity();
                Integer lastWeekCapacity = ObjectUtils.isEmpty(optimizeWeek.getLastWeekCapacity()) ? 0 : optimizeWeek.getLastWeekCapacity();
                Double congestIndex = ObjectUtils.isEmpty(optimizeWeek.getCongestIndex()) ? 0.0 : optimizeWeek.getCongestIndex();
                Double lastWeekCongestIndex = ObjectUtils.isEmpty(optimizeWeek.getLastWeekCongestIndex()) ? 0.0 : optimizeWeek.getLastWeekCongestIndex();
                Double speed = ObjectUtils.isEmpty(optimizeWeek.getSpeed()) ? 0.0 : optimizeWeek.getSpeed();
                Double lastWeekSpeed = ObjectUtils.isEmpty(optimizeWeek.getLastWeekSpeed()) ? 0.0 : optimizeWeek.getLastWeekSpeed();
                Integer travelTime = ObjectUtils.isEmpty(optimizeWeek.getTravelTime()) ? 0 : optimizeWeek.getTravelTime();
                Integer lastWeekTravelTime = ObjectUtils.isEmpty(optimizeWeek.getLastWeekTravelTime()) ? 0 : optimizeWeek.getLastWeekTravelTime();
                allCapacity += capacity;
                allCapacityLast += lastWeekCapacity;
                allCongestIndex += congestIndex;
                allCongestIndexLast += lastWeekCongestIndex;
                allSpeed += speed;
                allSpeedLast += lastWeekSpeed;
                allTravelTime += travelTime;
                allTravelTimeLast += lastWeekTravelTime;
                timeList.add(getTimeString(optimizeWeek));
                analysisDataVO.setWeekName(WeekEnum.getDesc(optimizeWeek.getWeekDay()));
                analysisDataVO.setOptimizeTimeSpan(getTimeString(optimizeWeek));
                analysisDataVO.setThisWeekCapacity(capacity);
                analysisDataVO.setLastWeekCapacity(lastWeekCapacity);
                analysisDataVO.setThisWeekCongestIndex(Math.round(congestIndex * 100) / 100.0);
                analysisDataVO.setLastWeekCongestIndex(Math.round(lastWeekCongestIndex * 100) / 100.0);
                analysisDataVO.setThisWeekAvgSpeed(Math.round(speed * 100) / 100.0);
                analysisDataVO.setLastWeekAvgSpeed(Math.round(lastWeekSpeed * 100) / 100.0);
                analysisDataVO.setThisWeekTravelTime(Math.round(travelTime * 10 / 60) / 10.0);
                analysisDataVO.setLastWeekTravelTime(Math.round(lastWeekTravelTime * 10 / 60) / 10.0);
                analysisDataVOS.add(analysisDataVO);
            }

            analysisResponseVO.setDataList(analysisDataVOS);
            analysisResponseVO.setOptimizeSpan(TimeIntervalMerger.timeString(timeList));
            analysisResponseVO.setAvgCapacity(String.valueOf(allCapacity / waveOptimizeWeeks.size()));
            analysisResponseVO.setAvgCongestIndex(String.valueOf(Math.round(allCongestIndex / waveOptimizeWeeks.size() * 100) / 100.0));
            analysisResponseVO.setAvgSpeed(String.valueOf(Math.round(allSpeed / waveOptimizeWeeks.size() * 100) / 100.0));
            analysisResponseVO.setAvgTravelTime(String.valueOf(allTravelTime / waveOptimizeWeeks.size()));
            analysisResponseVO.setAvgCapacityCompareSituation(allCapacity > allCapacityLast ? "提升" : "下降");
            double avgCongestIndex = allCongestIndex / waveOptimizeWeeks.size() - allCongestIndexLast / waveOptimizeWeeks.size();
            analysisResponseVO.setAvgCongestIndexCompare(avgCongestIndex >= 0 ? "增加" + String.valueOf(Math.round(avgCongestIndex * 100) / 100.0) + "%" : "减少" + String.valueOf(Math.round(Math.abs(avgCongestIndex) * 100) / 100.0) + "%");
            double avgSpeed = allSpeed / waveOptimizeWeeks.size() - allSpeedLast / waveOptimizeWeeks.size();
            analysisResponseVO.setAvgSpeedCompare(avgSpeed > 0 ? "提升" + String.valueOf(Math.round(avgSpeed * 100) / 100.0) + "%" : "降低" + String.valueOf(Math.round(Math.abs(avgSpeed) * 100) / 100.0) + "%");
            int avgTravelTime = allTravelTime / waveOptimizeWeeks.size() - allTravelTimeLast / waveOptimizeWeeks.size();
            analysisResponseVO.setAvgTravelTimeCompare(String.valueOf(avgTravelTime > 0 ? "增加" + String.valueOf(Math.round(avgTravelTime * 100) / 100.0) + "%" : "减少" + String.valueOf(Math.round(Math.abs(avgTravelTime) * 100) / 100.0) + "%"));
            //本周总体情况
            if (allCapacity / waveOptimizeWeeks.size() >= 0 && avgTravelTime <= 0) {
                if (allCapacity / waveOptimizeWeeks.size() >= 5) {
                    analysisResponseVO.setWeekTotalitySituation("明显提升");
                } else {
                    analysisResponseVO.setWeekTotalitySituation("略有提升");
                }
            } else if (allCapacity / waveOptimizeWeeks.size() >= 0 && avgTravelTime > 0) {
                if (allCapacity / waveOptimizeWeeks.size() - avgTravelTime > 0) {
                    analysisResponseVO.setWeekTotalitySituation("略有提升");
                } else {
                    analysisResponseVO.setWeekTotalitySituation("略有下降");
                }
            } else if (allCapacity / waveOptimizeWeeks.size() < 0 && avgTravelTime <= 0) {
                if (allCapacity / waveOptimizeWeeks.size() - avgTravelTime > 0) {
                    analysisResponseVO.setWeekTotalitySituation("略有提升");
                } else {
                    analysisResponseVO.setWeekTotalitySituation("略有下降");
                }
            } else if (allCapacity / waveOptimizeWeeks.size() < 0 && avgTravelTime > 0) {
                if (Math.abs(allCapacity / waveOptimizeWeeks.size()) >= 5) {
                    analysisResponseVO.setWeekTotalitySituation("明显下降");
                } else {
                    analysisResponseVO.setWeekTotalitySituation("略有下降");
                }
            }
            analysisResponseVOS.add(analysisResponseVO);

        }
        return analysisResponseVOS;
    }

    private String getTimeString(AnalysisGreenWaveOptimizeWeekVO optimizeWeek) {
        String start = format.format(optimizeWeek.getOptimizeStartTime());
        String end = format.format(optimizeWeek.getOptimizeEndTime());
        return start + "~" + end;
    }

}
