package net.wanji.opt.servicev2.report.impl;


import com.alibaba.fastjson.JSONObject;
import net.wanji.common.gts.service.GtsService;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.BaseAreaInfoMapper;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.databus.po.BaseAreaInfoPO;
import net.wanji.databus.po.BaseCrossInfoPO;

import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.vo.CrossInfoVO;
import net.wanji.opt.servicev2.report.BaseAreaInfoService;
import org.apache.dubbo.config.annotation.Service;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 区域基础信息
 * </p>
 *
 * @Author wangtao
 * @Date 2025-03-18
 */
@Slf4j
@Component
@Service
public class BaseAreaInfoServiceImpl   implements BaseAreaInfoService {

    @Resource
    private BaseAreaInfoMapper baseAreaInfoMapper;
    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private BaseCrossInfoMapper baseCrossInfoMapper;

    @Resource
    private GtsService gtsService;

    /**
     * 根据区划类型查询对应集合（含子集）
     * @param types
     * @return
     */
    public List<BaseAreaInfoPO> selectByType(String types) {
        List<BaseAreaInfoPO> retList = new ArrayList<>();

        String[] sps = types.split(",");
        for (String type : sps) {
            //指定查询一期示范区
            List<BaseAreaInfoPO> parentList = baseAreaInfoMapper.selectByType(Integer.valueOf(type));
            for (BaseAreaInfoPO parent : parentList) {
                List<BaseAreaInfoPO> child = baseAreaInfoMapper.selectByParentCode(parent.getId());
                retList.add(parent);
                //屏蔽子区输出
                //retList.addAll(child);
            }
        }

        return  retList;
    }

    @Override
    public JSONObject getGreenByWkt(Integer areaId, String wkt, Integer type) {
        if(ObjectUtils.isEmpty(wkt)){
            BaseAreaInfoPO areaInfoPO = baseAreaInfoMapper.selectById(areaId);
            wkt = areaInfoPO.getPolylines();
        }
        if(ObjectUtils.isEmpty(wkt)){
            return null;
        }
        JSONObject jsonObject = new JSONObject();
        if(ObjectUtils.isEmpty(type) || type == 1){
            String finalWkt = wkt;
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll().stream().filter(x->{
                //使用gts方法判断区域是否包含
                boolean contains = gtsService.contains(finalWkt, x.getWkt());
                if(contains){
                    return contains;
                }
                //使用gts方法判断区域是否相交
                boolean intersects = gtsService.intersects(finalWkt, x.getWkt());
                return intersects;
            }).collect(Collectors.toList());
            jsonObject.put("greenInfoList",greenwaveInfoPOList);
        }
        if(ObjectUtils.isEmpty(type) || type == 2){
            String finalWkt1 = wkt;
            CrossInfoVO crossInfoVO = new CrossInfoVO();
            crossInfoVO.setIsSignal(1);
            List<BaseCrossInfoPO> baseCrossInfoPOS = baseCrossInfoMapper.selectAll(crossInfoVO).stream().filter(x->{

                boolean contains = gtsService.contains(finalWkt1, convertPointString(x.getLocation()));
                return contains;
            }).collect(Collectors.toList());
            jsonObject.put("crossInfoList",baseCrossInfoPOS);
        }
        return jsonObject;
    }

    @Override
    public void insertAreaInfo(BaseAreaInfoPO baseAreaInfoPO) {
        baseAreaInfoPO.setCode(99);
        baseAreaInfoPO.setRoadName("自定义区域");
        baseAreaInfoPO.setLocation(gtsService.centroid(baseAreaInfoPO.getPolylines()));
        baseAreaInfoPO.setParentCode(99);
        baseAreaInfoPO.setRemark("");
        baseAreaInfoMapper.insertOne(baseAreaInfoPO);
    }

    @Override
    public void updateAreaNameAndRemark(String name, String remark, Integer id, String roadName, Integer parentCode, String wkt) {
        baseAreaInfoMapper.updateAreaNameAndRemark(name,remark,id,roadName,parentCode,wkt);
    }

    @Override
    public void deleteById(Integer areaId) {
        baseAreaInfoMapper.deleteById(areaId);
    }

    /**
     * 将 POINT 类型的字符串转换为经纬度格式
     *
     * @param pointStr 输入的 POINT 字符串，例如 "POINT(116.875165 36.648904)"
     * @return 转换后的经纬度字符串，例如 "116.875165,36.648904"
     */
    public static String convertPointString(String pointStr) {
        if (pointStr == null || !pointStr.startsWith("POINT(") || !pointStr.endsWith(")")) {
            throw new IllegalArgumentException("输入字符串格式不正确，必须是 POINT(x y) 格式");
        }

        // 去掉前缀 "POINT(" 和后缀 ")"
        String coordinates = pointStr.substring(6, pointStr.length() - 1);

        // 替换空格为逗号
        return coordinates.replace(" ", ",");
    }

}
