package net.wanji.opt.servicev2.report.impl;

import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.opt.common.enums.CrossDirEnum;
import net.wanji.opt.common.enums.CrossTypeEnum;
import net.wanji.opt.common.enums.LaneTurnEnum;
import net.wanji.opt.dao.mapper.report.CrossRidLaneMapper;
import net.wanji.opt.entity.report.CrossRidLaneDTO;
import net.wanji.opt.servicev2.report.CrossRidLaneService;
import net.wanji.opt.vo2.report.CrossPeakAnalysis;
import net.wanji.opt.vo2.report.CrossRidLaneResult;
import net.wanji.opt.vo2.report.CrossRidLaneVO;
import net.wanji.opt.vo2.report.CrossRunStateAnalysisResultResponseDTO;
import net.wanji.opt.vo2.report.CrossRunStateDetailAnalysisDataDTO;
import net.wanji.opt.vo2.report.LaneDirNumVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CrossRidLaneServiceImpl implements CrossRidLaneService {

    @Autowired
    private CrossRidLaneMapper crossRidLaneMapper;
    @Autowired
    private BaseCrossInfoMapper baseCrossInfoMapper;

    @Override
    public List<CrossRidLaneResult> getCrossInfoList(String crossIds) {
        List<String> crossIdList = null;
        if (ObjectUtil.isNotEmpty(crossIds)) {
            crossIdList = Arrays.stream(crossIds.split(","))
                    .map(String::trim) // 去除可能的空格
                    .filter(s -> !s.isEmpty()) // 过滤掉空字符串
                    .collect(Collectors.toList());
        }
        List<CrossRidLaneDTO> crossRidLaneDTOList = crossRidLaneMapper.getCrossInfoList(crossIdList);
        Map<String, List<CrossRidLaneDTO>> crossIdMap = crossRidLaneDTOList.stream().collect(Collectors.groupingBy(CrossRidLaneDTO::getCrossId));
        List<CrossRidLaneResult> crossRidLaneResultList = new ArrayList<>();
        for (List<CrossRidLaneDTO> crossRidLaneDTOS : crossIdMap.values()) {
            Map<String, List<CrossRidLaneDTO>> laneNameMap = crossRidLaneDTOS.stream().collect(Collectors.groupingBy(x -> {
                if (ObjectUtil.isNotEmpty(x.getLaneName())) {
                    String[] split = x.getLaneName().split(":");
                    return split[0];
                }
                String[] strings = x.getCrossName().split("与");
                return strings[0];
            }));
            CrossRidLaneResult crossRidLaneResult = new CrossRidLaneResult();
            crossRidLaneResult.setCrossId(crossRidLaneDTOS.get(0).getCrossId());
            crossRidLaneResult.setCrossName(crossRidLaneDTOS.get(0).getCrossName());
            crossRidLaneResult.setType(CrossTypeEnum.getDesc(crossRidLaneDTOS.get(0).getType()));
            crossRidLaneResult.setLocation(getLocation(crossRidLaneDTOS.get(0).getLocation()));

            List<CrossRidLaneVO> crossRidLaneVOS = new ArrayList<>();
            for (String laneName : laneNameMap.keySet()) {
                List<CrossRidLaneDTO> laneDTOS = laneNameMap.get(laneName);
                CrossRidLaneVO crossRidLaneVO = new CrossRidLaneVO();
                crossRidLaneVO.setLaneName(laneName);
                if (ObjectUtil.isEmpty(laneDTOS.get(0).getInDir()) || ObjectUtil.isEmpty(laneDTOS.get(0).getOutDir())) {
                    crossRidLaneVO.setLaneDir(getLaneDir(laneDTOS.get(0).getDir()));
                } else {
                    crossRidLaneVO.setLaneDir(CrossDirEnum.getDesc(laneDTOS.get(0).getInDir()) + CrossDirEnum.getDesc(laneDTOS.get(0).getOutDir()) + "走向");
                }
                crossRidLaneVO.setLaneNum(laneDTOS.size());
                Map<Integer, List<CrossRidLaneDTO>> dirLaneMap = laneDTOS.stream().collect(Collectors.groupingBy(CrossRidLaneDTO::getDir));
                List<LaneDirNumVO> laneDirNumVOS = new ArrayList<>();
                for (Integer dir : dirLaneMap.keySet()) {
                    List<CrossRidLaneDTO> dirLaneDTOS = dirLaneMap.get(dir);
                    LaneDirNumVO laneDirNumVO = new LaneDirNumVO();
                    laneDirNumVO.setLaneInDir(CrossDirEnum.getDesc(dir) + "进口");
                    laneDirNumVO.setLaneDirNum(dirLaneDTOS.size());
                    Map<Integer, List<CrossRidLaneDTO>> turnMap = dirLaneDTOS.stream().collect(Collectors.groupingBy(CrossRidLaneDTO::getTurn));
                    String laneDirTurn = "";
                    for (Integer turn : turnMap.keySet()) {
                        if ("".equals(laneDirTurn)) {
                            laneDirTurn = turnMap.get(turn).size() + LaneTurnEnum.getDesc(turn);
                        } else {
                            laneDirTurn = laneDirTurn + "," + turnMap.get(turn).size() + LaneTurnEnum.getDesc(turn);
                        }
                    }
                    laneDirNumVO.setLaneTurnNum(laneDirTurn);
                    laneDirNumVOS.add(laneDirNumVO);
                }
                crossRidLaneVO.setLaneDirList(laneDirNumVOS);
                crossRidLaneVOS.add(crossRidLaneVO);
            }
            crossRidLaneResult.setCrossRidLaneList(crossRidLaneVOS);
            crossRidLaneResultList.add(crossRidLaneResult);
        }
        return crossRidLaneResultList;
    }

    @Override
    public CrossRunStateAnalysisResultResponseDTO getTotalitySituation(String crossIds, Integer year, Integer week) {
        List<String> crossIdList = null;
        Integer yearWeek = Integer.valueOf("" + year + week);
        Integer yearWeekLast = Integer.valueOf("" + year + (week - 1));
        if (ObjectUtil.isNotEmpty(crossIds)) {
            crossIdList = Arrays.stream(crossIds.split(","))
                    .map(String::trim) // 去除可能的空格
                    .filter(s -> !s.isEmpty()) // 过滤掉空字符串
                    .collect(Collectors.toList());
        }
        CrossRunStateAnalysisResultResponseDTO crossRunStateAnalysisResultResponseDTO = new CrossRunStateAnalysisResultResponseDTO();
        //按周，路口id查询
        List<CrossPeakAnalysis> list = crossRidLaneMapper.getTotalitySituation(crossIdList, yearWeek, null, null);
        List<CrossPeakAnalysis> listLast = crossRidLaneMapper.getTotalitySituation(crossIdList, yearWeekLast, null, null);
        if (ObjectUtil.isEmpty(list)) {
            return null;
        }
        //最大车流路口
        String maxFlowCrossId = "";
        int maxFlowTotal = 0;
        //最大交通指数路口
        String maxTrafficIndexCrossId = "";
        double maxTrafficIndex = 0;
        for (CrossPeakAnalysis crossPeakAnalysis : list) {
            if (crossPeakAnalysis.getDayTotalFlow() > maxFlowTotal) {
                maxFlowTotal = crossPeakAnalysis.getDayTotalFlow();
                maxFlowCrossId = crossPeakAnalysis.getCrossId();
            }
        }
        BaseCrossInfoPO maxFlowTotalBaseCrossInfoPO = maxFlowTotalBaseCrossInfoPO = baseCrossInfoMapper.selectById(maxFlowCrossId);
        crossRunStateAnalysisResultResponseDTO.setMaxFlowCrossName(maxFlowTotalBaseCrossInfoPO.getName());
        for (CrossPeakAnalysis crossPeakAnalysis : list) {
            if (crossPeakAnalysis.getPeakTrafficIndex() > maxTrafficIndex) {
                maxTrafficIndex = crossPeakAnalysis.getDayTotalFlow();
                maxTrafficIndexCrossId = crossPeakAnalysis.getCrossId();
            }
        }
        BaseCrossInfoPO maxTrafficIndexBaseCrossInfoPO = baseCrossInfoMapper.selectById(maxTrafficIndexCrossId);
        crossRunStateAnalysisResultResponseDTO.setMaxTrafficIndexCrossName(maxTrafficIndexBaseCrossInfoPO.getName());

        //按工作日分组
        Map<Integer, List<CrossPeakAnalysis>> dayTypeMap = list.stream().collect(Collectors.groupingBy(CrossPeakAnalysis::getDayType));
        Map<Integer, List<CrossPeakAnalysis>> dayTypeMapLast = listLast.stream().collect(Collectors.groupingBy(CrossPeakAnalysis::getDayType));
        double workTotalFlow = 0;
        double workTotalFlowLast = 0;
        int workDayAmPeakMaxHourFlow = 0;
        int workDayAmPeakMaxHourFlowLast = 0;
        int workDayPmPeakMaxHourFlow = 0;
        int workDayPmPeakMaxHourFlowLast = 0;
        int workDayFmPeakMaxHourFlow = 0;
        int workDayFmPeakMaxHourFlowLast = 0;
        double weekEndAvgFlow = 0;
        double weekEndAvgFlowLast = 0;
        double workDayAmPeakAvgTrafficIndex = 0;
        double workDayAmPeakAvgTrafficIndexLast = 0;
        double workDayPmPeakAvgTrafficIndex = 0;
        double workDayPmPeakAvgTrafficIndexLast = 0;

        if (ObjectUtil.isNotEmpty(dayTypeMap.get(1))) {
            //工作日平均流量
            workTotalFlow = dayTypeMap.get(1).stream().mapToInt(CrossPeakAnalysis::getDayTotalFlow).average().orElse(0.0);
            //工作日高峰最大小时流量
            Map<Integer, Integer> peakMaxHourFlowByPeak = dayTypeMap.get(1).stream().collect(
                    Collectors.groupingBy(CrossPeakAnalysis::getDayType, // 按 dayType 分组
                            Collectors.collectingAndThen( // 对分组后的数据进行进一步处理
                                    Collectors.maxBy(Comparator.comparingInt(CrossPeakAnalysis::getPeakMaxHourFlow)), // 找到每组的最大值
                                    optionalAnalysis -> optionalAnalysis.map(CrossPeakAnalysis::getPeakMaxHourFlow).orElse(0) // 提取最大值并提供默认值
                            )));
            //工作日高峰平均交通指数
            Map<Integer, Double> workDayPeakAvgTrafficIndexBypeak = dayTypeMap.get(1).stream().collect(
                    Collectors.groupingBy(
                            CrossPeakAnalysis::getDayType, // 按 dayType 分组
                            Collectors.averagingDouble(CrossPeakAnalysis::getPeakTrafficIndex)
                    ));
            workDayAmPeakMaxHourFlow = ObjectUtil.isEmpty(peakMaxHourFlowByPeak.get(1)) ? 0 : peakMaxHourFlowByPeak.get(1);
            workDayPmPeakMaxHourFlow = ObjectUtil.isEmpty(peakMaxHourFlowByPeak.get(2)) ? 0 : peakMaxHourFlowByPeak.get(2);
            workDayFmPeakMaxHourFlow = ObjectUtil.isEmpty(peakMaxHourFlowByPeak.get(3)) ? 0 : peakMaxHourFlowByPeak.get(3);
            workDayAmPeakAvgTrafficIndex = ObjectUtil.isEmpty(workDayPeakAvgTrafficIndexBypeak.get(1)) ? 0 : workDayPeakAvgTrafficIndexBypeak.get(1);
            workDayPmPeakAvgTrafficIndex = ObjectUtil.isEmpty(workDayPeakAvgTrafficIndexBypeak.get(2)) ? 0 : workDayPeakAvgTrafficIndexBypeak.get(2);
        }
        if (ObjectUtil.isNotEmpty(dayTypeMapLast.get(1))) {
            //上周工作日平均流量
            workTotalFlowLast = dayTypeMapLast.get(1).stream().mapToInt(CrossPeakAnalysis::getDayTotalFlow).average().orElse(0.0);
            //上周工作日高峰最大小时流量
            Map<Integer, Integer> peakMaxHourFlowByPeakLast = dayTypeMapLast.get(1).stream().collect(
                    Collectors.groupingBy(CrossPeakAnalysis::getDayType, // 按 dayType 分组
                            Collectors.collectingAndThen( // 对分组后的数据进行进一步处理
                                    Collectors.maxBy(Comparator.comparingInt(CrossPeakAnalysis::getPeakMaxHourFlow)), // 找到每组的最大值
                                    optionalAnalysis -> optionalAnalysis.map(CrossPeakAnalysis::getPeakMaxHourFlow).orElse(0) // 提取最大值并提供默认值
                            )));
            //上周工作日高峰平均交通指数
            Map<Integer, Double> workDayPeakAvgTrafficIndexBypeakLast = dayTypeMapLast.get(1).stream().collect(
                    Collectors.groupingBy(
                            CrossPeakAnalysis::getDayType, // 按 dayType 分组
                            Collectors.averagingDouble(CrossPeakAnalysis::getPeakTrafficIndex)
                    ));
            workDayAmPeakMaxHourFlowLast = ObjectUtil.isEmpty(peakMaxHourFlowByPeakLast.get(1)) ? 0 : peakMaxHourFlowByPeakLast.get(1);
            workDayPmPeakMaxHourFlowLast = ObjectUtil.isEmpty(peakMaxHourFlowByPeakLast.get(2)) ? 0 : peakMaxHourFlowByPeakLast.get(2);
            workDayFmPeakMaxHourFlowLast = ObjectUtil.isEmpty(peakMaxHourFlowByPeakLast.get(3)) ? 0 : peakMaxHourFlowByPeakLast.get(3);
            workDayAmPeakAvgTrafficIndexLast = ObjectUtil.isEmpty(workDayPeakAvgTrafficIndexBypeakLast.get(1)) ? 0 : workDayPeakAvgTrafficIndexBypeakLast.get(1);
            workDayPmPeakAvgTrafficIndexLast = ObjectUtil.isEmpty(workDayPeakAvgTrafficIndexBypeakLast.get(2)) ? 0 : workDayPeakAvgTrafficIndexBypeakLast.get(2);
        }
        if (ObjectUtil.isNotEmpty(dayTypeMap.get(2))) {
            //周末车流量
            weekEndAvgFlow = dayTypeMap.get(2).stream().mapToInt(CrossPeakAnalysis::getDayTotalFlow).average().orElse(0.0);
        }
        if (ObjectUtil.isNotEmpty(dayTypeMapLast.get(2))) {
            //上周周末车流量
            weekEndAvgFlowLast = dayTypeMapLast.get(2).stream().mapToInt(CrossPeakAnalysis::getDayTotalFlow).average().orElse(0.0);
        }

        //工作日平均车流差
        double workTotalFlowSum = workTotalFlow - workTotalFlowLast;
        //周末平均车流差
        double weekEndAvgFlowSum = weekEndAvgFlow - weekEndAvgFlowLast;

        int workDayAmPeakMaxHourFlowSum = workDayAmPeakMaxHourFlow - workDayAmPeakMaxHourFlowLast;
        int workDayPmPeakMaxHourFlowSum = workDayPmPeakMaxHourFlow - workDayPmPeakMaxHourFlowLast;
        int workDayFmPeakMaxHourFlowSum = workDayFmPeakMaxHourFlow - workDayFmPeakMaxHourFlowLast;
        double workDayAmPeakAvgTrafficIndexSum = workDayAmPeakAvgTrafficIndex - workDayAmPeakAvgTrafficIndexLast;
        double workDayPmPeakAvgTrafficIndexSum = workDayPmPeakAvgTrafficIndex - workDayPmPeakAvgTrafficIndexLast;

        //工作日、周末车流
        crossRunStateAnalysisResultResponseDTO.setWorkDayAvgFlow(String.valueOf(Math.round(workTotalFlow)) + "辆");
        crossRunStateAnalysisResultResponseDTO.setWeekEndAvgFlow(String.valueOf(Math.round(weekEndAvgFlow)) + "辆");

        crossRunStateAnalysisResultResponseDTO.setWorkDayAmPeakMaxHourFlow(String.valueOf(workDayAmPeakMaxHourFlow) + "辆");
        crossRunStateAnalysisResultResponseDTO.setWorkDayPmPeakMaxHourFlow(String.valueOf(workDayPmPeakMaxHourFlow) + "辆");
        crossRunStateAnalysisResultResponseDTO.setWorkDayFmPeakMaxHourFlow(String.valueOf(workDayFmPeakMaxHourFlow) + "辆");
        crossRunStateAnalysisResultResponseDTO.setWorkDayAmPeakAvgTrafficIndex(String.valueOf(workDayAmPeakAvgTrafficIndex));
        crossRunStateAnalysisResultResponseDTO.setWorkDayPmPeakAvgTrafficIndex(String.valueOf(workDayPmPeakAvgTrafficIndex));

        if (workTotalFlowSum > 0) {
            crossRunStateAnalysisResultResponseDTO.setAvgFlowLastWeekCompare("提升" + Math.round(workTotalFlowSum) + "辆");
        } else {
            crossRunStateAnalysisResultResponseDTO.setAvgFlowLastWeekCompare("下降" + Math.abs(Math.round(workTotalFlowSum)) + "辆");
        }
        if (weekEndAvgFlowSum > 0) {
            crossRunStateAnalysisResultResponseDTO.setEndAvgFlowLastWeekCompare("提升" + weekEndAvgFlowSum + "辆");
        } else {
            crossRunStateAnalysisResultResponseDTO.setEndAvgFlowLastWeekCompare("下降" + Math.abs(weekEndAvgFlowSum) + "辆");
        }
        if (workDayAmPeakMaxHourFlowSum > 0) {
            crossRunStateAnalysisResultResponseDTO.setAmPeakMaxHourFlowLastWeekCompare("提升" + workDayAmPeakMaxHourFlowSum + "辆");
        } else {
            crossRunStateAnalysisResultResponseDTO.setAmPeakMaxHourFlowLastWeekCompare("下降" + Math.abs(workDayAmPeakMaxHourFlowSum) + "辆");
        }
        if (workDayPmPeakMaxHourFlowSum > 0) {
            crossRunStateAnalysisResultResponseDTO.setPmPeakMaxHourFlowLastWeekCompare("提升" + workDayPmPeakMaxHourFlowSum + "辆");
        } else {
            crossRunStateAnalysisResultResponseDTO.setPmPeakMaxHourFlowLastWeekCompare("下降" + Math.abs(workDayPmPeakMaxHourFlowSum) + "辆");
        }
        if (workDayFmPeakMaxHourFlowSum > 0) {
            crossRunStateAnalysisResultResponseDTO.setFmPeakMaxHourFlowLastWeekCompare("提升" + workDayFmPeakMaxHourFlowSum + "辆");
        } else {
            crossRunStateAnalysisResultResponseDTO.setFmPeakMaxHourFlowLastWeekCompare("下降" + Math.abs(workDayFmPeakMaxHourFlowSum) + "辆");
        }
        if (workDayAmPeakAvgTrafficIndexSum > 0) {
            if(workDayAmPeakAvgTrafficIndexLast == 0){
                crossRunStateAnalysisResultResponseDTO.setAmPeakAvgTrafficIndexLastWeekCompare("提升100%");
            }else {
                crossRunStateAnalysisResultResponseDTO.setAmPeakAvgTrafficIndexLastWeekCompare("提升" + Math.round(workDayAmPeakAvgTrafficIndexSum * 100 / workDayAmPeakAvgTrafficIndexLast * 100) / 100.0 + "%");
            }
        } else {
            if(workDayAmPeakAvgTrafficIndexLast == 0){
                crossRunStateAnalysisResultResponseDTO.setAmPeakAvgTrafficIndexLastWeekCompare("下降0%");
            }else {
                crossRunStateAnalysisResultResponseDTO.setAmPeakAvgTrafficIndexLastWeekCompare("下降" + Math.abs(Math.round(workDayAmPeakAvgTrafficIndexSum * 100 / workDayAmPeakAvgTrafficIndexLast * 100) / 100.0) + "%");
            }
        }
        if (workDayPmPeakAvgTrafficIndexSum > 0) {
            if(workDayAmPeakAvgTrafficIndexLast == 0){
                crossRunStateAnalysisResultResponseDTO.setPmPeakAvgTrafficIndexLastWeekCompare("提升100%");
            }else {
                crossRunStateAnalysisResultResponseDTO.setPmPeakAvgTrafficIndexLastWeekCompare("提升" + Math.round(workDayPmPeakAvgTrafficIndexSum * 100 / workDayAmPeakAvgTrafficIndexLast * 100) / 100.0 + "%");
            }
        } else {
            if(workDayAmPeakAvgTrafficIndexLast == 0){
                crossRunStateAnalysisResultResponseDTO.setPmPeakAvgTrafficIndexLastWeekCompare("下降0%");
            }else {
                crossRunStateAnalysisResultResponseDTO.setPmPeakAvgTrafficIndexLastWeekCompare("下降" + Math.abs(Math.round(workDayPmPeakAvgTrafficIndexSum * 100 / workDayPmPeakAvgTrafficIndexLast * 100) / 100.0) + "%");
            }
        }

        Map<String, List<CrossPeakAnalysis>> crossIdMap = list.stream().collect(Collectors.groupingBy(CrossPeakAnalysis::getCrossId));
        Map<String, List<CrossPeakAnalysis>> crossIdMapLast = listLast.stream().collect(Collectors.groupingBy(CrossPeakAnalysis::getCrossId));
        List<CrossRunStateDetailAnalysisDataDTO> crossRunStateDetailAnalysisDataDTOS = new ArrayList<>();
        if(ObjectUtil.isNotEmpty(crossIdMap)){
            for (String crossId : crossIdMap.keySet()) {
                CrossRunStateDetailAnalysisDataDTO crossRunStateDetailAnalysisDataDTO = new CrossRunStateDetailAnalysisDataDTO();
                List<CrossPeakAnalysis> crossPeakAnalyses = crossIdMap.get(crossId);
                List<CrossPeakAnalysis> crossPeakAnalysesLast = crossIdMapLast.get(crossId);
                int thisWeekPeakMaxFlow = 0;
                int lastWeekPeakMaxFlow = 0;
                int thisWeekEndMaxFlow = 0;
                int lastWeekEndMaxFlow = 0;
                double thisWeekWorkDayAmTrafficIndex = 0;
                int thisWeekWorkDayAmTrafficIndexCount = 0;
                double lastWeekWorkDayAmTrafficIndex = 0;
                int lastWeekWorkDayAmTrafficIndexCount = 0;
                double thisWeekWorkDayPmTrafficIndex = 0;
                int thisWeekWorkDayPmTrafficIndexCount = 0;
                double lastWeekWorkDayPmTrafficIndex = 0;
                int lastWeekWorkDayPmTrafficIndexCount = 0;
                if(ObjectUtil.isNotEmpty(crossPeakAnalyses)){
                    for (CrossPeakAnalysis crossPeakAnalysis : crossPeakAnalyses) {
                        if (crossPeakAnalysis.getDayType() == 1) {
                            if (crossPeakAnalysis.getPeakMaxHourFlow() > thisWeekPeakMaxFlow) {
                                thisWeekPeakMaxFlow = crossPeakAnalysis.getPeakMaxHourFlow();
                            }
                            if (crossPeakAnalysis.getPeakType() == 1) {
                                thisWeekWorkDayAmTrafficIndexCount++;
                                thisWeekWorkDayAmTrafficIndex += crossPeakAnalysis.getPeakTrafficIndex();
                            }
                            if (crossPeakAnalysis.getPeakType() == 2) {
                                thisWeekWorkDayPmTrafficIndexCount++;
                                thisWeekWorkDayPmTrafficIndex += crossPeakAnalysis.getPeakTrafficIndex();
                            }
                        }
                        if (crossPeakAnalysis.getDayType() == 2) {
                            if (crossPeakAnalysis.getPeakMaxHourFlow() > thisWeekEndMaxFlow) {
                                thisWeekEndMaxFlow = crossPeakAnalysis.getPeakMaxHourFlow();
                            }
                        }
                    }
                }
                if(ObjectUtil.isNotEmpty(crossPeakAnalysesLast)){
                    for (CrossPeakAnalysis crossPeakAnalysis : crossPeakAnalysesLast) {
                        if (crossPeakAnalysis.getDayType() == 1) {
                            if (crossPeakAnalysis.getPeakMaxHourFlow() > lastWeekPeakMaxFlow) {
                                lastWeekEndMaxFlow = crossPeakAnalysis.getPeakMaxHourFlow();
                            }
                            if (crossPeakAnalysis.getPeakType() == 1) {
                                lastWeekWorkDayAmTrafficIndexCount++;
                                lastWeekWorkDayAmTrafficIndex += crossPeakAnalysis.getPeakTrafficIndex();
                            }
                            if (crossPeakAnalysis.getPeakType() == 2) {
                                lastWeekWorkDayPmTrafficIndexCount++;
                                lastWeekWorkDayPmTrafficIndex += crossPeakAnalysis.getPeakTrafficIndex();
                            }
                        }
                        if (crossPeakAnalysis.getDayType() == 2) {
                            if (crossPeakAnalysis.getPeakMaxHourFlow() > lastWeekEndMaxFlow) {
                                lastWeekEndMaxFlow = crossPeakAnalysis.getPeakMaxHourFlow();
                            }
                        }
                    }
                }

                crossRunStateDetailAnalysisDataDTO.setThisWeekPeakMaxFlow(thisWeekPeakMaxFlow);
                crossRunStateDetailAnalysisDataDTO.setLastWeekPeakMaxFlow(lastWeekPeakMaxFlow);
                crossRunStateDetailAnalysisDataDTO.setThisWeekEndMaxFlow(thisWeekEndMaxFlow);
                crossRunStateDetailAnalysisDataDTO.setLastWeekEndMaxFlow(lastWeekEndMaxFlow);

                if(thisWeekWorkDayAmTrafficIndexCount==0){
                    crossRunStateDetailAnalysisDataDTO.setThisWeekWorkDayAmTrafficIndex(0.0);
                }else {
                    crossRunStateDetailAnalysisDataDTO.setThisWeekWorkDayAmTrafficIndex(thisWeekWorkDayAmTrafficIndex / thisWeekWorkDayAmTrafficIndexCount);
                }
                if(lastWeekWorkDayAmTrafficIndexCount==0){
                    crossRunStateDetailAnalysisDataDTO.setLastWeekWorkDayAmTrafficIndex(0.0);
                }else {
                    crossRunStateDetailAnalysisDataDTO.setLastWeekWorkDayAmTrafficIndex(lastWeekWorkDayAmTrafficIndex / lastWeekWorkDayAmTrafficIndexCount);
                }
                if(thisWeekWorkDayPmTrafficIndexCount==0){
                    crossRunStateDetailAnalysisDataDTO.setThisWeekWorkDayPmTrafficIndex(0.0);
                }else {
                    crossRunStateDetailAnalysisDataDTO.setThisWeekWorkDayPmTrafficIndex(thisWeekWorkDayPmTrafficIndex / thisWeekWorkDayPmTrafficIndexCount);
                }
                if(lastWeekWorkDayPmTrafficIndexCount==0){
                    crossRunStateDetailAnalysisDataDTO.setThisWeekWorkDayAmTrafficIndex(0.0);
                }else {
                    crossRunStateDetailAnalysisDataDTO.setThisWeekWorkDayAmTrafficIndex(lastWeekWorkDayPmTrafficIndex / lastWeekWorkDayPmTrafficIndexCount);
                }

                crossRunStateDetailAnalysisDataDTOS.add(crossRunStateDetailAnalysisDataDTO);
            }
        }

        crossRunStateAnalysisResultResponseDTO.setDataList(crossRunStateDetailAnalysisDataDTOS);
        return crossRunStateAnalysisResultResponseDTO;
    }

    private String getLaneDir(Integer dir) {
        switch (dir) {
            case 1:
                return "南北走向";
            case 2:
                return "东北西南走向";
            case 3:
                return "东西走向";
            case 4:
                return "东南西北走向";
            case 5:
                return "南北走向";
            case 6:
                return "东北西南走向";
            case 7:
                return "东西走向";
            case 8:
                return "东南西北走向";
            default:
                return "东西走向";
        }
    }

    private String getLocation(String location) {
        // 提取经纬度
        try {
            // 去掉 "POINT(" 和 ")"
            String content = location.substring(6, location.length() - 1);

            // 按空格分割
            String[] parts = content.split("\\s+");

            if (parts.length == 2) {
                String longitude = parts[0]; // 经度
                String latitude = parts[1];  // 纬度

                return longitude + "," + latitude;
            } else {
                log.error("未找到匹配的 POINT 格式");
            }
        } catch (Exception e) {
            log.error("解析失败: " + e.getMessage());
        }
        return "";
    }
}