package net.wanji.opt.servicev2.strategy.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.strategy.StrategyPriorityMapper;
import net.wanji.opt.entity.comprehensivequery.CrossEntity;
import net.wanji.opt.entity.comprehensivequery.GreenAndCrossEntity;
import net.wanji.opt.entity.comprehensivequery.GreenTreeEntity;
import net.wanji.opt.entity.strategy.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import net.wanji.opt.entity.strategy.dto.StrategyPriorityGroup;
import net.wanji.opt.servicev2.strategy.StrategyPriorityService;
import net.wanji.opt.synthesis.pojo.StrategyFactoryEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
* <p>
* 策略优先级日计划详情表 服务实现类
* </p>
*
* @author fengyi
* @since 2025-03-28
*/
@Service
@Slf4j
 public class StrategyPriorityServiceImpl extends ServiceImpl<StrategyPriorityMapper, StrategyPriorityDailyInfo> implements StrategyPriorityService {

	 @Resource
	 private StrategyPriorityMapper strategyPriorityMapper;

	/**
	 * 策略路口列表
	 * @param
	 * @return
	 */
	@Override
	public  List<StrategyPriorityDailyInfo> pageStrategyCrossList() {
		 List<StrategyPriorityDailyInfo> allList= strategyPriorityMapper.pageStrategyCrossList();

		 return allList ;
	 }

	/**
	 * 策略名称
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getStrategyList() {
		List<StrategyFactoryEntity> strategyList= strategyPriorityMapper.getStrategyList();
		return strategyList;
	}
	/**
	 * 场景
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getSceneList(Integer type) {

		List<StrategyFactoryEntity> strategyList= strategyPriorityMapper.getSceneList(type);
		return strategyList;
	}
	/**
	 * 厂商
	 * @param
	 * @return
	 */
	@Override
	public List<StrategyFactoryEntity> getCompanyList() {
		List<StrategyFactoryEntity> strategyList=strategyPriorityMapper.getCompanyList();
		return strategyList;
	}
	/*
	* 策略优先级保存 路口 干线共用
	* */
	@Override
	@Transactional
	public void savePriority(List<StrategyPriorityGroup> dataList) {
		try {
			List<StrategyPriorityConfig> saveList = new ArrayList<>();
			for  (StrategyPriorityGroup group : dataList) {
				//Type : 1;路口2：干线
				Integer  typePd=group.getType();
				if(typePd==1) {
					String label = group.getLabel();
					Integer labelCode = group.getLabelCode();
					//1；路口2干线
					Integer type = group.getType();
					//路口id
					List<String> crossList = group.getCrossIds();
					//选中多个路口存多条数据
					for (String crossId : crossList) {
						//先判断表里有没有数据，有的话删除t_strategy_priority_config
						List<StrategyPriorityConfig> priorityConfig = strategyPriorityMapper.selectPriorityTable(crossId);
						if (priorityConfig.size() > 0) {
							strategyPriorityMapper.deletePriorityConfig(crossId);
						}

						if (CollectionUtils.isNotEmpty(group.getData())) {
							for (StrategyPriorityConfig item : group.getData()) {
								StrategyPriorityConfig strategyPriorityConfig = new StrategyPriorityConfig();
								strategyPriorityConfig.setCrossId(crossId);
								//分组id
								strategyPriorityConfig.setGroupId(labelCode);
								//分组名称
								strategyPriorityConfig.setGroupName(label);
								//策略编号
								strategyPriorityConfig.setStrategyNo(item.getStrategyNo());
								//优先级
								strategyPriorityConfig.setPriority(item.getPriority());
								//type
								strategyPriorityConfig.setType(type);
								saveList.add(strategyPriorityConfig);
							}
						}
					}
				}else if (typePd==2){
					String label = group.getLabel();
					Integer labelCode = group.getLabelCode();
					//1；路口2干线
					Integer type = group.getType();
					//干线id
					List<Integer> greenList = group.getGreenIds();
					//选中多个干线存多条数据
					for (Integer greenId : greenList) {
						//先判断表里有没有数据，有的话删除t_strategy_priority_config
						List<StrategyPriorityConfig> priorityConfig = strategyPriorityMapper.selectGreenPriorityTable(greenId);
						if (priorityConfig.size() > 0) {
							strategyPriorityMapper.deleteGreenPriorityConfig(greenId);
						}

						if (CollectionUtils.isNotEmpty(group.getData())) {
							for (StrategyPriorityConfig item : group.getData()) {
								StrategyPriorityConfig strategyPriorityConfig = new StrategyPriorityConfig();
								//干线id
								strategyPriorityConfig.setGreenId(greenId);
								//分组id
								strategyPriorityConfig.setGroupId(labelCode);
								//分组名称
								strategyPriorityConfig.setGroupName(label);
								//策略编号
								strategyPriorityConfig.setStrategyNo(item.getStrategyNo());
								//优先级
								strategyPriorityConfig.setPriority(item.getPriority());
								//type
								strategyPriorityConfig.setType(type);
								saveList.add(strategyPriorityConfig);
							}
						}
					}
				}
			}
			strategyPriorityMapper.savePriorityInsert(saveList);
		} catch (Exception e) {
			log.error("{} savePriority", this.getClass().getSimpleName(), e);
			throw  e;
		}
	}

	/*
	* 策略优先级路口跟干线数据回显
	* */
	@Override
	@Transactional
	public List<StrategyFactoryEntity> getPriorityData(String crossId,Integer greenId, Integer type) throws Exception {

		try {
			List<StrategyFactoryEntity> priorityList =new ArrayList<>();
			//type: 1路口 2干线
			if (type==1) {
				if (crossId.isEmpty()){
					throw new IllegalArgumentException("crossId cannot be null");
				}
				//先获取策略编号
				List<String> strategyNo = strategyPriorityMapper.getstrategyNo(crossId,type);
				if (CollectionUtils.isEmpty(strategyNo)) {
					return Collections.emptyList();
				}
				 priorityList = strategyPriorityMapper.getPriorityConfigData(strategyNo, crossId,type);

			}else if (type==2){
				if (greenId==null){
					throw new IllegalArgumentException("greenId cannot be null");
				}
				//先获取策略编号
				List<String> strategyNo = strategyPriorityMapper.getGreenstrategyNo(greenId,type);
				if (CollectionUtils.isEmpty(strategyNo)) {
					return Collections.emptyList();
				}
				 priorityList = strategyPriorityMapper.getGreenPriorityConfigData(strategyNo, greenId,type);
			}
			return  priorityList;
		}catch (Exception e){
			log.error("{} getPriorityData", this.getClass().getSimpleName(), e);
			throw  e;
		}

	}

	/*
	* 策略管理日计划批量保存 干线跟路口
	* */
	@Override
	@Transactional
	public void savePlanConfig(List<StrategyPriorityGroup> dailyPlanDetails) throws JsonProcessingException {

		try {
			List<StrategyPriorityDailyInfo> savePlanList = new ArrayList<>();
			ObjectMapper objectMapper = new ObjectMapper();
			for (StrategyPriorityGroup group : dailyPlanDetails) {
				//Type : 1;路口2：干线
				Integer  typePd=group.getType();
				if (typePd==1) {
					//路口ID
					List<String> crossList = group.getCrossIds();
					//选中多个路口存多条数据
					for (String crossId : crossList) {
						StrategyPriorityDailyInfo strategyPriorityDailyInfo = new StrategyPriorityDailyInfo();
						strategyPriorityDailyInfo.setCrossId(crossId);
						//1；路口2干线
						Integer type = group.getType();
						strategyPriorityDailyInfo.setType(type);
						//日计划编号
						Integer dailyPlanId = group.getDailyPlanId();
						strategyPriorityDailyInfo.setDailyPlanId(dailyPlanId);
						//执行时间
						String weekExecute = group.getWeekExecute();
						strategyPriorityDailyInfo.setWeekExecute(weekExecute);
						//先判断表里有没有数据，有的话删除t_strategy_priority_daily_info
						List<StrategyPriorityDailyInfo> planConfig = strategyPriorityMapper.selectPlanTable(crossId);
						if (planConfig.size() > 0) {
							strategyPriorityMapper.deletePlanConfig(crossId);
						}

						//取出日计划详情
						List<StrategyPriorityPlanDetails> dailyPlanDetails1 = group.getDailyPlanDetails();
						String dailyPlanDetailsJson = JSON.toJSONString(dailyPlanDetails1);
						strategyPriorityDailyInfo.setDailyPlanDetails(dailyPlanDetailsJson);

						savePlanList.add(strategyPriorityDailyInfo);
					}
				}else if(typePd==2){
					//干线ID
					List<Integer> greenIdList = group.getGreenIds();
					//选中多个路口存多条数据
					for (Integer greenId : greenIdList) {
						StrategyPriorityDailyInfo strategyPriorityDailyInfo = new StrategyPriorityDailyInfo();
						strategyPriorityDailyInfo.setGreenId(greenId);
						//1；路口2干线
						Integer type = group.getType();
						strategyPriorityDailyInfo.setType(type);
						//日计划编号
						Integer dailyPlanId = group.getDailyPlanId();
						strategyPriorityDailyInfo.setDailyPlanId(dailyPlanId);
						//执行时间
						String weekExecute = group.getWeekExecute();
						strategyPriorityDailyInfo.setWeekExecute(weekExecute);
						//先判断表里有没有数据，有的话删除t_strategy_priority_daily_info
						List<StrategyPriorityDailyInfo> planConfig = strategyPriorityMapper.selectGreenPlanTable(greenId);
						if (planConfig.size() > 0) {
							strategyPriorityMapper.deleteGreenPlanConfig(greenId);
						}

						//取出日计划详情
						List<StrategyPriorityPlanDetails> dailyPlanDetails1 = group.getDailyPlanDetails();
						String dailyPlanDetailsJson = JSON.toJSONString(dailyPlanDetails1);
						strategyPriorityDailyInfo.setDailyPlanDetails(dailyPlanDetailsJson);

						savePlanList.add(strategyPriorityDailyInfo);
				}
				}
			}
			strategyPriorityMapper.savePlanConfig(savePlanList);

		}catch (Exception e){
			log.error("{} savePlanConfig", this.getClass().getSimpleName(), e);
			throw  e;
		}
	}
	/*
	* 日计划回显接口
	* */
	@Override
	public List<StrategyPriorityDailyInfo> getPlanConfigData(String crossId,Integer greenId,Integer type) {

       //type: 1路口 2干线
		List<StrategyPriorityDailyInfo> planList =new ArrayList<>();
		if (type==1){
			if (crossId.isEmpty()){
				throw new IllegalArgumentException("crossId cannot be null");
			}
			 planList = strategyPriorityMapper.getPlanConfigData(crossId,type);
		}else if (type==2){
			if (greenId==null){
				throw new IllegalArgumentException("greenId cannot be null");
			}
			planList = strategyPriorityMapper.getGreenPlanConfigData(greenId,type);
		}

		return planList;
	}
/*
*  策略参数配置
* */
	@Override
	@Transactional
	public void saveParamterConfig(StrategyPriorityGroup group) throws JsonProcessingException {

		try {
			List<StrategyParameterConfig> savePlanList = new ArrayList<>();
			//路口ID
			List<String> crossList = group.getCrossIds();
			//选中多个路口存多条数据
			for (String crossId : crossList) {

				StrategyParameterConfig strategyParameterConfig = new StrategyParameterConfig();
				//策略编号
				String strategyNo = group.getStrategyNo();
				strategyParameterConfig.setStrategyNo(strategyNo);
				//路口id
				//String crossId = group.getCrossId();
				strategyParameterConfig.setCrossId(crossId);
				//先判断表里有没有数据，有的话删除t_strategy_priority_parameter
 				List<StrategyParameterConfig> paramterConfig = strategyPriorityMapper.paramterConfigTable(crossId);
				if (paramterConfig.size() > 0) {
					strategyPriorityMapper.deleteParamterConfig(crossId);
				}
				//参数 场景配置
				List<StrategyPriorityParamter> parameterConfigList = group.getParameterConfigList();
				String jsonString = JSON.toJSONString(parameterConfigList);
				strategyParameterConfig.setParamDetails(jsonString);
				//策略调度参数配置
				StrategySchedulingParam schedulingParamters = group.getSchedulingParamters();
				ObjectMapper objectMapper = new ObjectMapper();
				String schedulingParamter = objectMapper.writeValueAsString(schedulingParamters);
				strategyParameterConfig.setSchedulingParam(schedulingParamter);
				//添加到最终数据
				savePlanList.add(strategyParameterConfig);
			}
				strategyPriorityMapper.saveParamConfig(savePlanList);

		}catch (Exception e){
			log.error("{} saveParamterConfig", this.getClass().getSimpleName(), e);
			throw  e;
	}

	}
	/*
	 *  策略参数配置数据回显
	 * */
	@Override
	public List<StrategyParameterConfig> getParamConfigData(String crossId) {
         try {
			List<StrategyParameterConfig> paraterConfigList = strategyPriorityMapper.getParamConfigData(crossId);
			return paraterConfigList;
		}catch (Exception e){
			log.error("{} getParamConfigData", this.getClass().getSimpleName(), e);
			throw  e;
		}

	}
	/*
	 *  策略干线列表
	 * */
	@Override
	public List<StrategyPriorityDailyInfo> getStrategyGreenWave() {

		try {
		List<StrategyPriorityDailyInfo> queryResult= strategyPriorityMapper.getStrategyGreenWave();

		Map<Integer, List<StrategyPriorityDailyInfo>> groupedData = queryResult.stream()
				.collect(Collectors.groupingBy(StrategyPriorityDailyInfo::getGreenId));

		// 格式化输出
		List<StrategyPriorityDailyInfo> result = groupedData.entrySet().stream()
				.map(entry -> {
					//干线ID
					int trunkId = entry.getKey();
					List<StrategyPriorityDailyInfo> infos = entry.getValue();
					// 创建新的对象
					StrategyPriorityDailyInfo strategyPriorityDailyInfo=new StrategyPriorityDailyInfo();
					// 提取干线名称（假设同一干线的名称相同）
					String trunkName = infos.get(0).getWaveName();


					// 按路口 ID 分组并合并日计划详情
					Map<String, Set<String>> crossGroupedDetails = infos.stream()
							.filter(info -> info.getDailyPlanDetails() != null)
							.collect(Collectors.groupingBy(
									StrategyPriorityDailyInfo::getCrossId,
									Collectors.mapping(StrategyPriorityDailyInfo::getDailyPlanDetails, Collectors.toSet()))); // 去重

					// 合并每个路口的日计划详情
					String dailyPlanDetails = crossGroupedDetails.values().stream()
							.map(set -> String.join(", ", set))
							.collect(Collectors.joining(", "));
					// 提取路口 ID 和路口名称
					String roadIds = infos.stream()
							.map(StrategyPriorityDailyInfo::getCrossId)
							.collect(Collectors.joining(", "));
					String roadNames = infos.stream()
							.map(StrategyPriorityDailyInfo::getCrossName)
							.collect(Collectors.joining(", "));
					strategyPriorityDailyInfo.setGreenId(trunkId);
					strategyPriorityDailyInfo.setWaveName(trunkName);
					strategyPriorityDailyInfo.setCrossId(roadIds);
					strategyPriorityDailyInfo.setCrossName(roadNames);
                    strategyPriorityDailyInfo.setDailyPlanDetails(dailyPlanDetails);
					return strategyPriorityDailyInfo;
				})
				.collect(Collectors.toList());
			// 输出结果
			return result;
		}catch (Exception e) {
			log.error("{} getStrategyGreenWave", this.getClass().getSimpleName(), e);
			throw  e;
		}

	}
}
