package net.wanji.opt.task;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.servicev2.evaluation.EventInfoService;
import net.wanji.opt.servicev2.judgeanalysis.AnalysisGreenCongestionPeriodService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.text.ParseException;

@Component
@Configurable
@EnableScheduling
@Slf4j
@Profile("docker")
public class AnalysisGreenCongestionPeriodWeekTask {
    @Autowired
    private AnalysisGreenCongestionPeriodService analysisGreenCongestionPeriodService;

    @Resource
    EventInfoService eventInfoService;

    @Scheduled(cron = "0 30 2 ? * 1")
    public  void task() throws ParseException {

        try {
            analysisGreenCongestionPeriodService.selectCountByCongestionPeriod();

            analysisGreenCongestionPeriodService.selectCountByLandData();

            //analysisGreenCongestionPeriodService.selectCountByCrossData();
        }catch (Exception e){
            log.error("",e);
        }
    }

    @Scheduled(cron = "0 0 2 ? * ? ")
    public  void taskDay() throws ParseException {

        try {
            analysisGreenCongestionPeriodService.selectCountByCrossDataOneDay();
        }catch (Exception e){
            log.error("",e);
        }
    }

    @Scheduled(cron = "0 0 2 ? * ? ")
    public  void insertCrossSlowRunEventData() throws ParseException {
        try {

            DateTime endTDateTime = DateUtil.date();
            //开始时间
            DateTime startTDateTime = DateUtil.offsetDay(endTDateTime, -1);
            startTDateTime = DateUtil.beginOfDay(startTDateTime);
            String sdt = startTDateTime.toString("yyyy-MM-dd HH:00:00");

            //窗口截止时间
            endTDateTime = DateUtil.parseDateTime(sdt).offset(DateField.HOUR,24).offset(DateField.SECOND,-1);
            String edt = endTDateTime.toString("yyyy-MM-dd HH:mm:ss");

            eventInfoService.insertCrossSlowRunEventData(sdt,edt);

        }catch (Exception e){
            log.error("",e);
        }
    }
}
