package net.wanji.opt.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenCrossDirDTO;
import net.wanji.databus.dao.entity.GreenwaveRealtimePO;
import net.wanji.databus.dao.mapper.GreenwaveRealtimeMapper;
import net.wanji.opt.cache.GreenCrossDirInfoCache;
import net.wanji.opt.common.CommonUtils;
import net.wanji.opt.common.RedisUtils;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.dao.mapper.GreenChartSchemeHistMapper;
import net.wanji.opt.dao.mapper.GreenChartSchemeRingPhaseMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.dto.GreenBeltChartDTO;
import net.wanji.opt.dto.RingPhaseInfoDTO;
import net.wanji.opt.dto.SchemeGreenDirGreenTimeDTO;
import net.wanji.opt.entity.GreenChartSchemeHist;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.vo.GreenBeltInfoVO;
import net.wanji.opt.vo2.dto.GreenLastOptResultDTO;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2025/04/01 11:18
 * @description 海信通过reids缓存方案相位信息，设置绿波时序图
 */
@Component
@Slf4j
@Profile("docker")
public class HisenseGreenChartTask {

    @Resource
    private RedisUtils redisUtils;
    @Resource
    private GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    @Resource
    private GreenChartSchemeHistMapper greenChartSchemeHistMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    private GreenChartSchemeRingPhaseMapper greenChartSchemeRingPhaseMapper;


    @Scheduled(fixedRate = 10 * 60 * 1000, initialDelay = 1 * 60 * 1000)
    public void getGreenChart() {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            Map<String, GreenwaveRealtimePO> realTimeMap = getGreenRealTimeMap();
            // 绿波路口数据
            List<GreenCrossDirDTO> greenCrossDirCache = GreenCrossDirInfoCache.greenCrossDirCache;
            List<SchemeGreenDirGreenTimeDTO> schemeGreenDirGreenTimeDTOS = getSchemeGreenDirGreenTimeDTOS(greenCrossDirCache);
            Map<Integer, List<SchemeGreenDirGreenTimeDTO>> greenSchemeRingPhaseMap = new HashMap<>(10);
            if (!CollectionUtils.isEmpty(greenCrossDirCache) && !CollectionUtils.isEmpty(schemeGreenDirGreenTimeDTOS)) {
                // 将海信方案环相位转成map形式
                HashMap<String, SchemeGreenDirGreenTimeDTO> greenDirGreenTimeMap = new HashMap<>();
                for (SchemeGreenDirGreenTimeDTO dto : schemeGreenDirGreenTimeDTOS) {
                    String crossId = dto.getCrossId();
                    Integer dir = dto.getDir();
                    greenDirGreenTimeMap.put(CommonUtils.getStrJoin(crossId, String.valueOf(dir)), dto);
                    // 存储海信方案环图信息
                    for (GreenCrossDirDTO greenCrossDirDTO : greenCrossDirCache) {
                        Integer greenId = greenCrossDirDTO.getGreenId();
                        String greenCrossId = greenCrossDirDTO.getCrossId();
                        if (StringUtils.equalsIgnoreCase(greenCrossId, crossId)) {
                            List<SchemeGreenDirGreenTimeDTO> list1 = greenSchemeRingPhaseMap.get(greenId);
                            if (CollectionUtils.isEmpty(list1)) {
                                List<SchemeGreenDirGreenTimeDTO> list2 = new ArrayList<>();
                                list2.add(dto);
                                greenSchemeRingPhaseMap.put(greenId, list2);
                            } else {
                                list1.add(dto);
                                greenSchemeRingPhaseMap.put(greenId, list1);
                            }
                        }
                    }
                }

                // 1.根据绿波带长度和速度计算行程时间
                // 2.根据相位差，行程时间，海信环图计算绿灯红灯点位 正向 crossRedTimesMap 反向backCrossRedTimesMap
                // todo test
                //List<GreenCrossDirDTO> collect = greenCrossDirCache.stream().filter(dto -> dto.getGreenId() == 1).collect(Collectors.toList());

                Map<Integer, List<GreenCrossDirDTO>> greenMap = greenCrossDirCache.stream().collect(Collectors.groupingBy(GreenCrossDirDTO::getGreenId));
                List<GreenBeltChartDTO> greenBeltChartDTOS = new ArrayList<>();
                for (Map.Entry<Integer, List<GreenCrossDirDTO>> entry : greenMap.entrySet()) {
                    GreenBeltChartDTO greenBeltChartDTO = new GreenBeltChartDTO();
                    Integer greenId = entry.getKey();
                    List<GreenCrossDirDTO> value = entry.getValue();
                    // 1.获取行程时间
                    GreenCrossDirDTO indexDTO = null;
                    if (Objects.equals(3, greenId)) {
                        indexDTO = value.get(2);
                    } else {
                        indexDTO = value.get(0);
                    }
                    Integer sort = indexDTO.getSort();
                    Integer indexDir = indexDTO.getInDir();
                    Integer totalDistance = indexDTO.getTotalDistance();

                    String greenKey = CommonUtils.getStrJoin(String.valueOf(greenId), String.valueOf(indexDir));
                    GreenwaveRealtimePO greenwaveRealtimePO = realTimeMap.get(greenKey);
                    Double speed = Objects.isNull(greenwaveRealtimePO.getSpeed()) ? 0 : greenwaveRealtimePO.getSpeed();
                    // km/h速度换成m/s
                    Double travelTime = totalDistance / speed * 3.6;
                    List<Double> offsets = new ArrayList<>();
                    List<Double> distances = new ArrayList<>();
                    // 正向红灯时间
                    Map<String, List<List<Double>>> crossRedTimesMap = new LinkedHashMap<>();
                    Map<String, List<List<Double>>> crossGreenTimesMap = new LinkedHashMap<>();
                    getCrossRedTimeMap(mapper, greenDirGreenTimeMap, value, indexDir, travelTime, offsets, distances, crossRedTimesMap, crossGreenTimesMap);

                    // 反向红灯时间
                    Integer indexOutDir = indexDTO.getOutDir();
                    List<GreenCrossDirDTO> backValue = value.stream().sorted(Comparator.comparingInt(GreenCrossDirDTO::getSort).reversed()).collect(Collectors.toList());
                    List<Double> backOffsets = new ArrayList<>();
                    List<Double> backDistances = new ArrayList<>();

                    String backGreenKey = CommonUtils.getStrJoin(String.valueOf(greenId), String.valueOf(indexOutDir));
                    GreenwaveRealtimePO backGreenwaveRealtimePO = realTimeMap.get(backGreenKey);
                    // km/h速度换成m/s
                    Double backSpeed = Objects.isNull(backGreenwaveRealtimePO.getSpeed()) ? 0 : backGreenwaveRealtimePO.getSpeed();
                    Double backTravelTime = totalDistance / backSpeed * 3.6;
                    Map<String, List<List<Double>>> backCrossRedTimesMap = new LinkedHashMap<>();
                    Map<String, List<List<Double>>> backCrossGreenTimesMap = new LinkedHashMap<>();
                    getCrossRedTimeMap(mapper, greenDirGreenTimeMap, backValue, indexOutDir, backTravelTime, backOffsets, backDistances, backCrossRedTimesMap, backCrossGreenTimesMap);
                    greenBeltChartDTO.setGreenId(greenId);
                    greenBeltChartDTO.setDir(indexDir);
                    greenBeltChartDTO.setBackDir(indexOutDir);
                    greenBeltChartDTO.setSpeed(speed);
                    greenBeltChartDTO.setBackSpeed(backSpeed);
                    greenBeltChartDTO.setOffsets(offsets);
                    greenBeltChartDTO.setBackOffsets(backOffsets);
                    greenBeltChartDTO.setDistances(distances);
                    greenBeltChartDTO.setBackDistances(backDistances);
                    greenBeltChartDTO.setCrossRedTimesMap(crossRedTimesMap);
                    greenBeltChartDTO.setBackCrossRedTimesMap(backCrossRedTimesMap);
                    greenBeltChartDTO.setCrossGreenTimesMap(crossGreenTimesMap);
                    greenBeltChartDTO.setBackCrossGreenTimesMap(backCrossGreenTimesMap);
                    greenBeltChartDTOS.add(greenBeltChartDTO);
                }

                // 3.计算每两个路口绿灯开始、结束时间
                List<GreenBeltChartDTO> results = new ArrayList<>();
                for (Map.Entry<Integer, List<GreenCrossDirDTO>> entry : greenMap.entrySet()) {
                    List<Map<String, List<List<Double>>>> greenwaveStartGroup = new ArrayList<>();
                    List<Map<String, List<List<Double>>>> backGreenwaveStartGroup = new ArrayList<>();
                    Integer greenId = entry.getKey();
                    List<GreenCrossDirDTO> value = entry.getValue();
                    // 正向 1
                    getGreenwaveStartGroup(greenBeltChartDTOS, greenwaveStartGroup, greenId, value, 1);
                    // 反向 2
                    List<GreenCrossDirDTO> backValue = value.stream().sorted(Comparator.comparingInt(GreenCrossDirDTO::getSort).reversed()).collect(Collectors.toList());
                    getGreenwaveStartGroup(greenBeltChartDTOS, backGreenwaveStartGroup, greenId, backValue, 2);
                    greenBeltChartDTOS.forEach(item -> {
                        if (Objects.equals(greenId, item.getGreenId())) {
                            Map<String, List<List<Double>>> backCrossRedTimesMap = item.getBackCrossRedTimesMap();
                            List<Map.Entry<String, List<List<Double>>>> entries = new ArrayList<>(backCrossRedTimesMap.entrySet());
                            Collections.reverse(entries);
                            Map<String, List<List<Double>>> reverseMap = entries.stream()
                                    .collect(Collectors.toMap(
                                            Map.Entry::getKey,
                                            Map.Entry::getValue,
                                            (e1, e2) -> e1,
                                            LinkedHashMap::new
                                    ));
                            item.setBackCrossRedTimesMap(reverseMap);
                            item.setGreenwaveStartGroup(greenwaveStartGroup);
                            item.setBackGreenwaveStartGroup(backGreenwaveStartGroup);
                            results.add(item);
                        }
                    });
                }
                for (GreenBeltChartDTO result : results) {
                    Integer greenId = result.getGreenId();
                    List<SchemeGreenDirGreenTimeDTO> list = greenSchemeRingPhaseMap.get(greenId);
                    GreenChartSchemeHist greenChartSchemeHist = new GreenChartSchemeHist();
                    greenChartSchemeHist.setGreenId(greenId);
                    greenChartSchemeHist.setGreenChart(mapper.writeValueAsString(result));
                    greenChartSchemeHist.setSchemeDetail(mapper.writeValueAsString(list));
                    Date date = null;
                    Integer fromScheme = 1;
                    if (!CollectionUtils.isEmpty(list)) {
                        SchemeGreenDirGreenTimeDTO dto = list.get(0);
                        date = dto.getDate();
                        if (Objects.nonNull(dto.getFromScheme())) {
                            fromScheme = dto.getFromScheme();
                        }
                    }
                    greenChartSchemeHist.setQueryDate(date);
                    greenChartSchemeHist.setFromScheme(fromScheme);
                    greenChartSchemeHistMapper.insert(greenChartSchemeHist);
                }
                log.debug("绿波图实体参数：{}", mapper.writeValueAsString(results));
                log.debug("绿波路口方案环图信息：{}", mapper.writeValueAsString(greenSchemeRingPhaseMap));
            }
        } catch (Exception e) {
            log.error("通过海信方案处理绿波波定时任务失败：{}", e);
            //throw new RuntimeException(e);
        }
    }

    private static void getGreenwaveStartGroup(List<GreenBeltChartDTO> greenBeltChartDTOS, List<Map<String, List<List<Double>>>> greenwaveStartGroup,
                                               Integer greenId, List<GreenCrossDirDTO> value, int type) {
        if (!CollectionUtils.isEmpty(value)) {
            for (int i = 0; i < value.size(); i++) {
                Map<String, List<List<Double>>> greenStartMap = new LinkedHashMap<>();
                List<List<Double>> firstGreenStartList = new ArrayList<>();
                List<List<Double>> secondGreenStartList = new ArrayList<>();
                List<Double> firstStartEndList = new ArrayList<>();
                GreenCrossDirDTO firstDto = value.get(i);
                String firstCrossId = firstDto.getCrossId();
                Integer inCrossDir = null;
                if (1 == type) {
                    inCrossDir = firstDto.getInDir();
                }
                if (2 == type) {
                    inCrossDir = firstDto.getOutDir();
                }
                if (i + 1 < value.size()) {
                    GreenCrossDirDTO secondDto = value.get(i + 1);
                    String secondCrossId = secondDto.getCrossId();
                    for (GreenBeltChartDTO greenBeltChartDTO : greenBeltChartDTOS) {
                        Integer beltGreenId = greenBeltChartDTO.getGreenId();
                        Integer beltDir = null;
                        if (Objects.equals(3, greenId)) {
                            if (1 == type) {
                                beltDir = greenBeltChartDTO.getDir() - 2;
                            }
                            if (2 == type) {
                                beltDir = greenBeltChartDTO.getBackDir() - 2;
                            }
                        } else {
                            if (1 == type) {
                                beltDir = greenBeltChartDTO.getDir();
                            }
                            if (2 == type) {
                                beltDir = greenBeltChartDTO.getBackDir();
                            }
                        }
                        if (Objects.equals(greenId, beltGreenId) && Objects.equals(inCrossDir, beltDir)) {
                            Double travelTime = getTravelTime(i, greenBeltChartDTO, type);
                            Map<String, List<List<Double>>> crossGreenTimesMap = null;
                            if (1 == type) {
                                crossGreenTimesMap = greenBeltChartDTO.getCrossGreenTimesMap();
                            }
                            if (2 == type) {
                                crossGreenTimesMap = greenBeltChartDTO.getBackCrossGreenTimesMap();
                            }
                            List<List<Double>> firstList = crossGreenTimesMap.get(firstCrossId);
                            List<List<Double>> secondList = crossGreenTimesMap.get(secondCrossId);
                            // 第一个路口循环绿灯开始时间
                            if (!CollectionUtils.isEmpty(firstList) && !CollectionUtils.isEmpty(secondList)) {
                                for (int i1 = 0; i1 < firstList.size(); i1++) {
                                    List<Double> greenStartEnd = firstList.get(i1);
                                    Double greenStart = greenStartEnd.get(0);
                                    Double greenEnd = greenStartEnd.get(1);
                                    for (int j = greenStart.intValue(); j <= greenEnd.intValue(); j++) {
                                        int firstStartPoint = j + travelTime.intValue();
                                        for (int i3 = 0; i3 < secondList.size(); i3++) {
                                            List<Double> greenStartEnd2 = secondList.get(i3);
                                            Double greenStart2 = greenStartEnd2.get(0);
                                            Double greenEnd2 = greenStartEnd2.get(1);

                                            if (greenStart <= j && j <= greenEnd
                                                    && greenStart2 <= firstStartPoint && firstStartPoint <= greenEnd2) {

                                                Double temp = j * 1.0;
                                                firstStartEndList.add(temp);
                                            }
                                        }
                                    }
                                }
                            }
                            if (!CollectionUtils.isEmpty(firstStartEndList)) {
                                Double start = firstStartEndList.get(0);
                                Double end = firstStartEndList.get(0);
                                for (int k = 1; k < firstStartEndList.size(); k++) {
                                    Double next = firstStartEndList.get(k);
                                    Double prev = firstStartEndList.get(k - 1);
                                    if (next - prev == 1.0) {
                                        end = firstStartEndList.get(k);
                                        if (k + 1 == firstStartEndList.size()) {
                                            firstGreenStartList.add(Arrays.asList(start, end));
                                            secondGreenStartList.add(Arrays.asList(start + travelTime.intValue(), end + travelTime.intValue()));
                                        }
                                    } else {
                                        firstGreenStartList.add(Arrays.asList(start, end));
                                        secondGreenStartList.add(Arrays.asList(start + travelTime.intValue(), end + travelTime.intValue()));
                                        start = firstStartEndList.get(k);
                                        end = firstStartEndList.get(k);
                                    }
                                }
                            }
                            greenStartMap.put(firstCrossId, firstGreenStartList);
                            greenStartMap.put(secondCrossId, secondGreenStartList);
                        }
                    }
                }
                greenwaveStartGroup.add(greenStartMap);
            }
        }
    }

    @NotNull
    private static Double getTravelTime(int i, GreenBeltChartDTO greenBeltChartDTO, int type) {
        Double travelTime = 0.0;
        if (1 == type) {
            Double speed = greenBeltChartDTO.getSpeed();
            List<Double> distances = greenBeltChartDTO.getDistances();
            Double distance = distances.get(i);
            travelTime = distance / speed * 3.6;
        }
        if (2 == type) {
            Double speed = greenBeltChartDTO.getBackSpeed();
            List<Double> distances = greenBeltChartDTO.getBackDistances();
            Double distance = distances.get(i + 1);
            travelTime = distance / speed * 3.6;
        }
        return travelTime;
    }


    private static Map<String, List<List<Double>>> getCrossRedTimeMap(ObjectMapper mapper, HashMap<String, SchemeGreenDirGreenTimeDTO> greenDirGreenTimeMap,
                                                                      List<GreenCrossDirDTO> value, Integer indexDir, Double travelTime,
                                                                      List<Double> offsets, List<Double> distances, Map<String, List<List<Double>>> crossRedTimesMap,
                                                                      Map<String, List<List<Double>>> crossGreenTimesMap) throws Exception {
        for (GreenCrossDirDTO greenCrossDirDTO : value) {
            List<List<Double>> crossRedTimes = new ArrayList<>();
            List<List<Double>> crossGreenTimes = new ArrayList<>();
            String crossId = greenCrossDirDTO.getCrossId();
            Integer distance = greenCrossDirDTO.getDistance();
            distances.add(distance * 1.0);
            String crossKey = null;
            if (StringUtils.equalsIgnoreCase("13MQV0B5KE0", crossId) || StringUtils.equalsIgnoreCase("13MRN0B5MV0", crossId)) {
                crossKey = CommonUtils.getStrJoin(String.valueOf(crossId), String.valueOf(indexDir - 2));
            } else {
                crossKey = CommonUtils.getStrJoin(String.valueOf(crossId), String.valueOf(indexDir));
            }
            // 获取海信环相位信息
            SchemeGreenDirGreenTimeDTO firstGreenTimeDTO = greenDirGreenTimeMap.get(crossKey);
            if (Objects.isNull(firstGreenTimeDTO)) {
                log.error("当前路口：{}， 没有环相位信息异常", crossKey);
                continue;
            }
            Integer cycleLen = firstGreenTimeDTO.getCycleLen();
            // * 1.0转Double
            Double offset = firstGreenTimeDTO.getOffset() * 1.0;
            offsets.add(offset);

            Integer greenStartTime = firstGreenTimeDTO.getStart();
            Integer greenEndTime = firstGreenTimeDTO.getEnd();
            // 计算出绿灯结束时间到周期结束时间:
            Integer redEndTime = cycleLen - greenEndTime;
            // 通过行程时间计算周期数量
            //double cycleSum = Math.floor(travelTime / cycleLen) + 1;
            // 默认三个周期
            int cycleSum = 3;
            for (int i = 0; i <= cycleSum; i++) {
                Double cycleIndex = i * Double.valueOf(cycleLen);
                // 多个周期后，需要在最后补充的时间
                Double endCycleExt = cycleLen - offset;
                // 红灯结束时间 > 相位差, 相位差都是红灯,endCycleExt包含绿灯
                if (redEndTime >= offset) {
                    // offset+cycleLen 中红灯时间
                    if (i == 0) {
                        crossRedTimes.add(Arrays.asList(cycleIndex, cycleIndex + offset + greenStartTime));
                        crossRedTimes.add(Arrays.asList(cycleIndex + offset + greenEndTime, cycleLen + offset));
                        // 绿灯时间
                        crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime, cycleIndex + offset + greenEndTime));
                    } else if (i > 0 && i < cycleSum) {
                        crossRedTimes.add(Arrays.asList(cycleIndex + offset + greenEndTime, cycleLen * (i + 1) + offset));
                        // 绿灯时间
                        crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime, cycleIndex + offset + greenEndTime));
                    } else if (i == cycleSum) {
                        // a最后一个周期补充相位差
                        // 绿灯时间
                        if (endCycleExt <= greenStartTime) {
                            crossRedTimes.add(Arrays.asList(cycleIndex + offset, cycleIndex + offset + endCycleExt));
                        } else if (greenStartTime <= endCycleExt && endCycleExt <= greenEndTime) {
                            crossRedTimes.add(Arrays.asList(cycleIndex + offset, cycleIndex + offset + greenStartTime));
                            crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime, cycleIndex + offset + endCycleExt));
                        } else {
                            crossRedTimes.add(Arrays.asList(cycleIndex + offset, cycleIndex + offset + greenStartTime));
                            crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime, cycleIndex + offset + greenEndTime));
                            crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenEndTime, cycleIndex + offset + endCycleExt));
                        }
                    }
                } else { // 红灯结束时间 < 相位差, 相位差包含绿灯,endCycleExt都是红灯
                    // 相位差中绿灯时间
                    if (i == 0) {
                        Double offsetGreen = greenEndTime - endCycleExt;
                        if (endCycleExt < greenStartTime) {
                            crossRedTimes.add(Arrays.asList(0.0, greenStartTime - endCycleExt));
                            crossGreenTimes.add(Arrays.asList(greenStartTime * 1.0 - endCycleExt, greenEndTime - endCycleExt));
                        }
                        crossRedTimes.add(Arrays.asList(offsetGreen, offset + greenStartTime));
                        crossRedTimes.add(Arrays.asList(cycleIndex + offset + greenEndTime, cycleLen + offset));
                        // 绿灯时间
                        crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime, cycleIndex + offset + greenEndTime));
                    } else if (i > 0 && i < cycleSum) {
                        crossRedTimes.add(Arrays.asList(cycleIndex + offset, cycleIndex + offset + greenStartTime));
                        crossRedTimes.add(Arrays.asList(cycleIndex + offset + greenEndTime, cycleLen * (i + 1) + offset));
                        // 绿灯时间
                        crossGreenTimes.add(Arrays.asList(cycleIndex + offset + greenStartTime, cycleIndex + offset + greenEndTime));
                    }
                    // 最后一个周期补充相位差
                    if (i == cycleSum) {
                        crossRedTimes.add(Arrays.asList(cycleIndex + offset, cycleIndex + offset + endCycleExt));
                        //crossRedTimes.add(Arrays.asList(cycleIndex + offset + greenEndTime, cycleIndex + offset + endCycleExt));
                    }

                }
            }
            crossRedTimesMap.put(crossId, crossRedTimes);
            crossGreenTimesMap.put(crossId, crossGreenTimes);
            log.debug("当前路口编号：{}，当前协调方向：{}， 红灯点位信息：{}", crossId, indexDir, mapper.writeValueAsString(crossRedTimes));
        }
        return crossRedTimesMap;
    }

    private Map<String, GreenwaveRealtimePO> getGreenRealTimeMap() {
        Map<String, GreenwaveRealtimePO> realTimeMap = new HashMap<>();
        LambdaQueryWrapper<GreenwaveRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        List<GreenwaveRealtimePO> greenwaveRealtimePOList = greenwaveRealtimeMapper.selectList(queryWrapper);
        for (GreenwaveRealtimePO greenwaveRealtimePO : greenwaveRealtimePOList) {
            Integer greenId = greenwaveRealtimePO.getGreenId();
            String roadDirection = greenwaveRealtimePO.getRoadDirection();
            Integer inDir = GreenBeltDirEnum.getInDir(roadDirection);
            realTimeMap.put(CommonUtils.getStrJoin(String.valueOf(greenId), String.valueOf(inDir)), greenwaveRealtimePO);
        }
        return realTimeMap;
    }

    /**
     * 通过绿波路口获取海信路口协调方向相位绿灯信息
     *
     * @return
     * @throws Exception
     */
    private List<SchemeGreenDirGreenTimeDTO> getSchemeGreenDirGreenTimeDTOS(List<GreenCrossDirDTO> greenCrossDirCache) throws Exception {
        List<SchemeGreenDirGreenTimeDTO> schemeGreenDirGreenTimeDTOS = new ArrayList<>();
        ObjectMapper mapper = JacksonUtils.getInstance();

        if (!CollectionUtils.isEmpty(greenCrossDirCache)) {
            for (GreenCrossDirDTO greenCrossDirDTO : greenCrossDirCache) {
                Integer greenId = greenCrossDirDTO.getGreenId();
                String greenCrossId = greenCrossDirDTO.getCrossId();
                String greenDirStr = null;
                if (Objects.equals(3, greenId)) {
                    greenDirStr = "1,3,5,7";
                } else {
                    greenDirStr = greenCrossDirDTO.getGreenDir();
                }

                // 查询神思绿波优化下发
                if (isGreenOptSend(greenId)) {
                    List<SchemeGreenDirGreenTimeDTO> greenOptList = getGreenOptList(mapper, greenId);
                    schemeGreenDirGreenTimeDTOS.addAll(greenOptList);
                    continue;
                }

                // 海信原始方案环相位信息
                String str = (String) redisUtils.getHash("utc-hisense-ring-phase", greenCrossId);
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                List<RingPhaseInfoDTO> ringPhaseInfoDTOS = mapper.readValue(str, new TypeReference<List<RingPhaseInfoDTO>>() {
                });
                // 遍历绿波路口协调方向
                if (StringUtils.isNotBlank(greenDirStr)) {
                    String[] greenDirArray = greenDirStr.split(",");
                    for (String s : greenDirArray) {
                        // 协调方向
                        Integer greenDir = Integer.valueOf(s);

                        if (!CollectionUtils.isEmpty(ringPhaseInfoDTOS)) {
                            Map<Integer, List<RingPhaseInfoDTO>> ringMap = ringPhaseInfoDTOS.stream().collect(Collectors.groupingBy(RingPhaseInfoDTO::getRingNo));
                            for (Map.Entry<Integer, List<RingPhaseInfoDTO>> entry : ringMap.entrySet()) {
                                Integer ringNo = entry.getKey();
                                if (Objects.equals(0, ringNo)) {
                                    continue;
                                }
                                List<RingPhaseInfoDTO> ringPhaseList = entry.getValue();
                                List<RingPhaseInfoDTO> sort = ringPhaseList.stream().sorted(Comparator.comparingInt(RingPhaseInfoDTO::getSort)).collect(Collectors.toList());
                                Integer start = 0;
                                SchemeGreenDirGreenTimeDTO schemeGreenDirGreenTimeDTO = new SchemeGreenDirGreenTimeDTO();
                                for (RingPhaseInfoDTO ringPhaseInfoDTO : sort) {
                                    List<RingPhaseInfoDTO.DirTurn> dirTurnInfoList = ringPhaseInfoDTO.getDirTurnInfoList();
                                    if (CollectionUtils.isEmpty(dirTurnInfoList)) {
                                        continue;
                                    }
                                    Integer phaseId = ringPhaseInfoDTO.getPhaseId();
                                    Integer greenTime = ringPhaseInfoDTO.getGreenTime();
                                    Integer yellowTime = ringPhaseInfoDTO.getYellowTime();
                                    Integer redTime = ringPhaseInfoDTO.getRedTime();
                                    Integer schemeNo = ringPhaseInfoDTO.getSchemeNo();
                                    Integer cycleLen = ringPhaseInfoDTO.getCycleLen();
                                    Integer offset = ringPhaseInfoDTO.getOffset();
                                    Date date = ringPhaseInfoDTO.getDate();
                                    for (RingPhaseInfoDTO.DirTurn dirTurn : dirTurnInfoList) {
                                        Integer dir = dirTurn.getDir();
                                        Integer turn = dirTurn.getTurn();
                                        // 如果路口信息与绿波协调方向相同
                                        // 如果是红灯，作为开始时间，结束时间是start + greenTime + yellowTime
                                        if (Objects.equals(dir, greenDir) && Objects.equals(2, turn)) {
                                            // 黄灯做绿灯处理
                                            schemeGreenDirGreenTimeDTO.setCrossId(greenCrossId);
                                            schemeGreenDirGreenTimeDTO.setSchemeNo(schemeNo);
                                            schemeGreenDirGreenTimeDTO.setDir(dir);
                                            schemeGreenDirGreenTimeDTO.setPhaseId(phaseId);
                                            schemeGreenDirGreenTimeDTO.setStart(start);
                                            schemeGreenDirGreenTimeDTO.setRingNo(ringNo);
                                            schemeGreenDirGreenTimeDTO.setCycleLen(cycleLen);
                                            schemeGreenDirGreenTimeDTO.setOffset(offset);
                                            Integer endTime = start + greenTime + yellowTime;
                                            schemeGreenDirGreenTimeDTO.setEnd(endTime);
                                            schemeGreenDirGreenTimeDTO.setDate(date);
                                            schemeGreenDirGreenTimeDTO.setFromScheme(1);
                                            schemeGreenDirGreenTimeDTOS.add(schemeGreenDirGreenTimeDTO);
                                        } else { // 如果是非协调方向， 相位时间作为开始时间
                                            start += greenTime + yellowTime + redTime;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            //log.error("路口绿灯协调方向时间：{}", mapper.writeValueAsString(schemeGreenDirGreenTimeDTOS));
        }
        return schemeGreenDirGreenTimeDTOS;
    }

    private List<SchemeGreenDirGreenTimeDTO> getGreenOptList(ObjectMapper mapper, Integer greenId) throws Exception {
        List<SchemeGreenDirGreenTimeDTO> schemeGreenDirGreenTimeDTOS = new ArrayList<>();
        List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectByGreenId(String.valueOf(greenId));
        if (!CollectionUtils.isEmpty(entities)) {
            for (StrategyGreenOptHistEntity entity : entities) {
                String crossGreenDetail = entity.getCrossGreenDetail();
                Integer cycle = entity.getCycle();
                String dir = entity.getDir();
                Integer inDir = GreenBeltDirEnum.getInDir(dir);
                Double greenWidthTime = entity.getGreenWidthTime();
                List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetailList = mapper.readValue(crossGreenDetail, new TypeReference<List<GreenBeltInfoVO.CrossGreenDetail>>() {
                });
                if (!CollectionUtils.isEmpty(crossGreenDetailList)) {
                    // 默认神思优化方案100，相位101
                    int index = 100;
                    for (GreenBeltInfoVO.CrossGreenDetail greenDetail : crossGreenDetailList) {
                        SchemeGreenDirGreenTimeDTO dto = new SchemeGreenDirGreenTimeDTO();
                        String crossId = greenDetail.getCrossId();
                        dto.setCrossId(crossId);
                        dto.setSchemeNo(100);
                        dto.setOffset(greenDetail.getOffset().intValue());
                        dto.setCycleLen(cycle);
                        dto.setRingNo(1);
                        ++index;
                        dto.setPhaseId(index);
                        Double greenStart = greenDetail.getPhaseStartTime();
                        dto.setStart(greenStart.intValue());
                        Double greenEnd = greenDetail.getPhaseEndTime();
                        dto.setEnd(greenEnd.intValue());
                        if (StringUtils.equalsIgnoreCase("13MQV0B5KE0", crossId) || StringUtils.equalsIgnoreCase("13MRN0B5MV0", crossId)) {
                            if (Objects.equals(3, inDir)) {
                                dto.setDir(1);
                            } else {
                                dto.setDir(7);
                            }
                        } else {
                            dto.setDir(inDir);
                        }
                        dto.setDate(new Date());
                        dto.setFromScheme(2);
                        schemeGreenDirGreenTimeDTOS.add(dto);
                        greenChartSchemeRingPhaseMapper.insert(dto);
                    }
                }
            }
        }
        return schemeGreenDirGreenTimeDTOS;
    }

    /**
     * 查询是否是海信绿波方案下发
     *
     * @return
     */
    private Boolean isGreenOptSend(Integer greenId) {
        List<GreenLastOptResultDTO> greenLastOptResultDTOS = strategyGreenOptHistMapper.selectLastGreenOptResultList();
        if (!CollectionUtils.isEmpty(greenLastOptResultDTOS)) {
            for (GreenLastOptResultDTO greenLastOptResultDTO : greenLastOptResultDTOS) {
                if (Objects.equals(greenId, greenLastOptResultDTO.getGreenId()) && Objects.equals(1, greenLastOptResultDTO.getControlMethod())) {
                    Date optTime = greenLastOptResultDTO.getOptTime();
                    LocalDateTime optLocal = optTime.toInstant().atZone(ZoneId.of("+8")).toLocalDateTime();
                    LocalDateTime optPlus15 = optLocal.plusMinutes(30);
                    LocalDateTime now = LocalDateTime.now();
                    if (optLocal.isBefore(now) && optPlus15.isAfter(now)) {
                        return Boolean.TRUE;
                    }
                }
            }
        }
        return Boolean.FALSE;
    }

}
