package net.wanji.opt.task;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenCrossDirDTO;
import net.wanji.opt.cache.GreenCrossDirInfoCache;
import net.wanji.opt.common.RedisUtils;
import net.wanji.opt.dao.mapper.GreenChartSchemeRingPhaseMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.dto.RingPhaseInfoDTO;
import net.wanji.opt.dto.SchemeGreenDirGreenTimeDTO;
import net.wanji.opt.vo2.dto.GreenLastOptResultDTO;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2025/04/15 15:09
 */
@Component
@Slf4j
@Profile("docker")
public class HisenseSchemeRingPhaseInfoTask {

    @Resource
    private RedisUtils redisUtils;
    @Resource
    private GreenChartSchemeRingPhaseMapper greenChartSchemeRingPhaseMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;

    @Scheduled(fixedRate = 10 * 60 * 1000, initialDelay = 1 * 60 * 1000)
    public void getSchemeRingPhase() {
        List<SchemeGreenDirGreenTimeDTO> schemeGreenDirGreenTimeDTOS = new ArrayList<>();
        ObjectMapper mapper = JacksonUtils.getInstance();
        try {
            List<GreenCrossDirDTO> greenCrossDirCache = GreenCrossDirInfoCache.greenCrossDirCache;
            if (!CollectionUtils.isEmpty(greenCrossDirCache)) {
                for (GreenCrossDirDTO greenCrossDirDTO : greenCrossDirCache) {
                    Integer greenId = greenCrossDirDTO.getGreenId();
                    String greenCrossId = greenCrossDirDTO.getCrossId();
                    String greenDirStr = null;
                    if (Objects.equals(3, greenId)) {
                        greenDirStr = "1,3,5,7";
                    } else {
                        greenDirStr = greenCrossDirDTO.getGreenDir();
                    }
                    // 神思下发绿波时不需要优化
                    if (isGreenOptSend(greenId)) {
                        continue;
                    }

                    // 海信环相位信息
                    String str = (String) redisUtils.getHash("utc-hisense-ring-phase", greenCrossId);
                    if (StringUtils.isBlank(str)) {
                        continue;
                    }
                    List<RingPhaseInfoDTO> ringPhaseInfoDTOS = mapper.readValue(str, new TypeReference<List<RingPhaseInfoDTO>>() {});
                    // 遍历绿波路口协调方向
                    if (StringUtils.isNotBlank(greenDirStr)) {
                        String[] greenDirArray = greenDirStr.split(",");
                        for (String s : greenDirArray) {
                            // 协调方向
                            Integer greenDir = Integer.valueOf(s);

                            if (!CollectionUtils.isEmpty(ringPhaseInfoDTOS)) {
                                Map<Integer, List<RingPhaseInfoDTO>> ringMap = ringPhaseInfoDTOS.stream().collect(Collectors.groupingBy(RingPhaseInfoDTO::getRingNo));
                                for (Map.Entry<Integer, List<RingPhaseInfoDTO>> entry : ringMap.entrySet()) {
                                    Integer ringNo = entry.getKey();
                                    if (Objects.equals(0, ringNo)) {
                                        continue;
                                    }
                                    List<RingPhaseInfoDTO> ringPhaseList = entry.getValue();
                                    List<RingPhaseInfoDTO> sort = ringPhaseList.stream().sorted(Comparator.comparingInt(RingPhaseInfoDTO::getSort)).collect(Collectors.toList());
                                    Integer start = 0;
                                    SchemeGreenDirGreenTimeDTO schemeGreenDirGreenTimeDTO = new SchemeGreenDirGreenTimeDTO();
                                    for (RingPhaseInfoDTO ringPhaseInfoDTO : sort) {
                                        List<RingPhaseInfoDTO.DirTurn> dirTurnInfoList = ringPhaseInfoDTO.getDirTurnInfoList();
                                        if (CollectionUtils.isEmpty(dirTurnInfoList)) {
                                            continue;
                                        }
                                        Integer phaseId = ringPhaseInfoDTO.getPhaseId();
                                        Integer greenTime = ringPhaseInfoDTO.getGreenTime();
                                        Integer yellowTime = ringPhaseInfoDTO.getYellowTime();
                                        Integer redTime = ringPhaseInfoDTO.getRedTime();
                                        Integer schemeNo = ringPhaseInfoDTO.getSchemeNo();
                                        Integer cycleLen = ringPhaseInfoDTO.getCycleLen();
                                        Integer offset = ringPhaseInfoDTO.getOffset();
                                        Date date = ringPhaseInfoDTO.getDate();
                                        for (RingPhaseInfoDTO.DirTurn dirTurn : dirTurnInfoList) {
                                            Integer dir = dirTurn.getDir();
                                            Integer turn = dirTurn.getTurn();
                                            // 如果路口信息与绿波协调方向相同
                                            // 如果是红灯，作为开始时间，结束时间是start + greenTime + yellowTime
                                            if (Objects.equals(dir, greenDir) && Objects.equals(2, turn)) {
                                                // 黄灯做绿灯处理
                                                schemeGreenDirGreenTimeDTO.setCrossId(greenCrossId);
                                                schemeGreenDirGreenTimeDTO.setSchemeNo(schemeNo);
                                                schemeGreenDirGreenTimeDTO.setDir(dir);
                                                schemeGreenDirGreenTimeDTO.setPhaseId(phaseId);
                                                schemeGreenDirGreenTimeDTO.setStart(start);
                                                schemeGreenDirGreenTimeDTO.setRingNo(ringNo);
                                                schemeGreenDirGreenTimeDTO.setCycleLen(cycleLen);
                                                schemeGreenDirGreenTimeDTO.setOffset(offset);
                                                Integer endTime = start + greenTime + yellowTime;
                                                schemeGreenDirGreenTimeDTO.setEnd(endTime);
                                                schemeGreenDirGreenTimeDTO.setDate(date);
                                                schemeGreenDirGreenTimeDTO.setFromScheme(1);
                                                schemeGreenDirGreenTimeDTOS.add(schemeGreenDirGreenTimeDTO);
                                            } else { // 如果是非协调方向， 相位时间作为开始时间
                                                start += greenTime + yellowTime + redTime;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                //log.error("路口绿灯协调方向时间：{}", mapper.writeValueAsString(schemeGreenDirGreenTimeDTOS));
            }
            if (!CollectionUtils.isEmpty(schemeGreenDirGreenTimeDTOS)) {
                for (SchemeGreenDirGreenTimeDTO schemeGreenDirGreenTimeDTO : schemeGreenDirGreenTimeDTOS) {
                    greenChartSchemeRingPhaseMapper.insert(schemeGreenDirGreenTimeDTO);
                }
            }
        } catch (Exception e) {
            log.error("方向环相位入库失败：{}", e);
            //throw new RuntimeException(e);
        }
    }

    /**
     * 查询是否是海信绿波方案下发
     *
     * @return
     */
    private Boolean isGreenOptSend(Integer greenId) {
        List<GreenLastOptResultDTO> greenLastOptResultDTOS = strategyGreenOptHistMapper.selectLastGreenOptResultList();
        if (!CollectionUtils.isEmpty(greenLastOptResultDTOS)) {
            for (GreenLastOptResultDTO greenLastOptResultDTO : greenLastOptResultDTOS) {
                if (Objects.equals(greenId, greenLastOptResultDTO.getGreenId())
                        && Objects.equals(1, greenLastOptResultDTO.getControlMethod())) {
                    Date optTime = greenLastOptResultDTO.getOptTime();
                    LocalDateTime optLocal = optTime.toInstant().atZone(ZoneId.of("+8")).toLocalDateTime();
                    LocalDateTime optMinus15 = optLocal.minusMinutes(15);
                    LocalDateTime now = LocalDateTime.now();
                    if (optMinus15.isBefore(now) && optLocal.isAfter(now)) {
                        return Boolean.TRUE;
                    }
                }
            }
        }
        return Boolean.FALSE;
    }
}
