package net.wanji.opt.vo2.report.design;

import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import net.wanji.opt.vo2.ParentResult;

import java.util.ArrayList;
import java.util.List;

@Data
public class CrossImportProblemAnalysisResult extends ParentResult {

    private CrossImportProblemResultResponse content;

    public  CrossImportProblemResultResponse getContent() {
        return content;
    }

    public CrossImportProblemAnalysisResult setContent(CrossImportProblemResultResponse content) {
        this.content = content;
        return this;
    }

    @Override
    public String toString() {
        return JSON.toJSONString(this);
    }

}

@Data
class  CrossImportProblemResultResponse {

    @ApiModelProperty(value = "溢出最严重路口", example = "旅游路-洪山路交叉口、旅游路-霞景路交叉口",position = 0)
    private String overFlowCrossName;
    @ApiModelProperty(value = "本周溢出时长", example = "3.5小时",position = 1)
    private String overFlowDuration;

    @ApiModelProperty(value = "拥堵最严重路口", example = "旅游路-洪山路交叉口、旅游路-霞景路交叉口",position = 2)
    private String congestCrossName;
    @ApiModelProperty(value = "本周溢出时长", example = "3.5小时",position = 3)
    private String congestDuration;

    //================================================================================================//

    @ApiModelProperty(value = "溢出路口排名，top10",position = 4)
    private List<CrossImportProblemData> overFlowList=new ArrayList<>();

    @ApiModelProperty(value = "拥堵路口排名，top10",position = 5)
    private List<CrossImportProblemData> congestList=new ArrayList<>();

}

@Data
class CrossImportProblemData  {
    @ApiModelProperty(value = "数据类型：1溢出 2拥堵",example = "1",position = 0)
    private   String dataType ;

    @ApiModelProperty(value = "路口名称",example = "旅游路-洪山路交叉口",position = 1)
    private   String crossName ;

    @ApiModelProperty(value = "持续时长",example = "90",position = 2)
    private   Integer duration ;

    @ApiModelProperty(value = "本周路口最严重时段",example = "7:00~7:30",position = 3)
    private   String seriousTimeSpan ;
    @ApiModelProperty(value = "本周路口最严重时段持续时长",example = "溢出时长30分钟",position = 4)
    private   String seriousTimeSpanDuration ;

    @ApiModelProperty(value = "工作日平均每天溢出次数",example = "3次",position = 5)
    private   String workDayAvgOverCount ;
    @ApiModelProperty(value = "工作日平均每天溢出时长",example = "溢出时长30分钟",position = 6)
    private   String workDayAvgOverDuration ;

    @ApiModelProperty(value = "路口每天时段详情",position = 7)
    private List<CrossImportProblemDetail> list=new ArrayList<>();

}

@Data
class CrossImportProblemDetail{
    @ApiModelProperty(value = "周几",example = "周一",position = 0)
    private   String weekName ;
    @ApiModelProperty(value = "时段",example = "7:00~7:30",position = 1)
    private   String timeSpan ;
    @ApiModelProperty(value = "时长",example = "30分钟",position = 2)
    private   String duration ;
}




