package net.wanji.opt.vo2.report.design;

import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import net.wanji.opt.vo2.ParentResult;

import java.util.ArrayList;
import java.util.List;

@Data
public class CrossRunStatePeakAnalysisResult extends ParentResult {

    private List<CrossRunStatePeakResultResponse> content;

    public List<CrossRunStatePeakResultResponse> getContent() {
        return content;
    }

    public CrossRunStatePeakAnalysisResult setContent(List<CrossRunStatePeakResultResponse> content) {
        this.content = content;
        return this;
    }

    @Override
    public String toString() {
        return JSON.toJSONString(this);
    }

}

@Data
class CrossRunStatePeakResultResponse {

    @ApiModelProperty(value = "路口名称", example = "经十路（舜耕路-山大路）")
    private String crossName;

    @ApiModelProperty(value = "工作日早高峰平均交通指数与上周比较情况", example = "增加【减少】10%")
    private String amPeakAvgTrafficIndexLastWeekCompare;
    @ApiModelProperty(value = "工作日晚高峰平均交通指数与上周比较情况", example = "增加【减少】10%")
    private String pmPeakAvgTrafficIndexLastWeekCompare;

    @ApiModelProperty(value = "工作日早高峰通行能力与上周比较情况", example = "提升【下降】5%")
    private String amPeakCapacityLastWeekCompare;
    @ApiModelProperty(value = "工作日晚高峰通行能力与上周比较情况", example = "提升【下降】5%")
    private String pmPeakCapacityLastWeekCompare;

    @ApiModelProperty(value = "工作日早高峰平均延误与上周比较情况", example = "增加【减少】11%")
    private String amPeakDelayTimeLastWeekCompare;
    @ApiModelProperty(value = "工作日晚高峰平均延误与上周比较情况", example = "增加【减少】11%")
    private String pmPeakDelayTimeLastWeekCompare;

    @ApiModelProperty(value = "工作日早高峰最大排队与上周比较情况", example = "增加【减少】11%")
    private String amPeakMaxQueueLengthLastWeekCompare;
    @ApiModelProperty(value = "工作日晚高峰最大排队与上周比较情况", example = "增加【减少】11%")
    private String pmPeakMaxQueueLengthLastWeekCompare;

    @ApiModelProperty(value = "工作日早高峰停车次数与上周比较情况", example = "增加【减少】5%")
    private String amPeakStopTimesLastWeekCompare;
    @ApiModelProperty(value = "工作日晚高峰停车次数与上周比较情况", example = "增加【减少】5%")
    private String pmPeakStopTimesLastWeekCompare;

    @ApiModelProperty(value = "总体运行效果", example = "明显提升【略有下降】")
    private String totalityRunSituation;

    //================================================================================================//
    @ApiModelProperty(value = "本周早高峰集中时段", example = "7:30~8:30")
    private String thisWeekAmPeakTimeSpan;
    @ApiModelProperty(value = "本周早高峰集中时段持续时长", example = "45分钟")
    private String thisWeekAmPeakDuration;
    @ApiModelProperty(value = "本周早高峰时段开始时间上周比较情况", example = "提前【延后】20分钟")
    private String thisWeekAmPeakStartTimeSituation;
    @ApiModelProperty(value = "本周早高峰时段结束时间上周比较情况", example = "提前【延后】20分钟")
    private String thisWeekAmPeakEndTimeSituation;
    @ApiModelProperty(value = "本周早高峰持续时长上周比较情况", example = "提前【延后】20分钟")
    private String thisWeekAmPeakDurationSituation;

    @ApiModelProperty(value = "本周早高峰集中时段", example = "7:30~8:30")
    private String thisWeekPmPeakTimeSpan;
    @ApiModelProperty(value = "本周早高峰集中时段持续时长", example = "45分钟")
    private String thisWeekPmPeakDuration;
    @ApiModelProperty(value = "本周早高峰时段开始时间上周比较情况", example = "提前【延后】20分钟")
    private String thisWeekPmPeakStartTimeSituation;
    @ApiModelProperty(value = "本周早高峰时段结束时间上周比较情况", example = "提前【延后】20分钟")
    private String thisWeekPmPeakEndTimeSituation;
    @ApiModelProperty(value = "本周早高峰持续时长上周比较情况", example = "提前【延后】20分钟")
    private String thisWeekPmPeakDurationSituation;

    //================================================================================================//
    //峰期交通流参数对比分析
    private List<CrossRunStatePeakDetailAnalysisData> dataList = new ArrayList<>();
    //高峰期时段分析
    private List<CrossRunStatePeakTimeSpanAnalysisData> peakList = new ArrayList<>();

}

@Data
class CrossRunStatePeakDetailAnalysisData {

    @ApiModelProperty(value = "峰期名称", example = "早高峰/晚高峰",position = 0)
    private String peakName;

    @ApiModelProperty(value = "本周交通指数", example = "3.4",position = 1)
    private Integer thisWeekTrafficIndex;
    @ApiModelProperty(value = "上周交通指数", example = "3.4",position = 2)
    private Integer lastWeekTrafficIndex;

    @ApiModelProperty(value = "本周通行能力", example = "15900",position = 3)
    private Integer thisWeekWorkCapacity;
    @ApiModelProperty(value = "上周通行能力", example = "16000",position = 4)
    private Integer lastWeekCapacity;

    @ApiModelProperty(value = "本周平均延误（S）", example = "40",position = 5)
    private Integer thisWeekWorkDelayTime;
    @ApiModelProperty(value = "上周平均延误（S）", example = "40",position = 6)
    private Integer lastWeekDelayTime;

    @ApiModelProperty(value = "本周最大排队", example = "240",position = 7)
    private Integer thisWeekMaxQueueLength;
    @ApiModelProperty(value = "上周最大排队", example = "240",position = 8)
    private Integer lastWeekMaxQueueLength;

    @ApiModelProperty(value = "本周停车次数", example = "4",position = 9)
    private Integer thisWeekStopTimes;
    @ApiModelProperty(value = "上周停车次数", example = "3.5",position = 10)
    private Integer lastWeekStopTimes;

}

@Data
class CrossRunStatePeakTimeSpanAnalysisData {

    @ApiModelProperty(value = "星期", example = "周一",position = 0)
    private String weekName;

    @ApiModelProperty(value = "本周早高峰时段", example = "7:00~8:00",position = 1)
    private String thisWeekAmPeakTimeSpan;
    @ApiModelProperty(value = "上周早高峰时段", example = "7:00~8:00",position = 2)
    private String lastWeekAmPeakTimeSpan;

    @ApiModelProperty(value = "本周早高峰持续时长（min）", example = "30",position = 3)
    private String thisWeekAmPeakDuration;
    @ApiModelProperty(value = "上周早高峰持续时长（min）", example = "30",position = 4)
    private String lastWeekAmPeakDuration;

    @ApiModelProperty(value = "本周晚高峰时段", example = "7:00~8:00",position = 5)
    private String thisWeekPmPeakTimeSpan;
    @ApiModelProperty(value = "上周晚高峰时段", example = "7:00~8:00",position = 6)
    private String lastWeekPmPeakTimeSpan;

    @ApiModelProperty(value = "本周晚高峰持续时长（min）", example = "30",position = 7)
    private String thisWeekPmPeakDuration;
    @ApiModelProperty(value = "上周晚高峰持续时长（min）", example = "30",position = 8)
    private String lastWeekPmPeakDuration;

}

