package net.wanji.opt.vo2.report.design;

import com.alibaba.fastjson.JSON;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import net.wanji.opt.vo2.ParentResult;

import java.util.ArrayList;
import java.util.List;

@Data
public class ImportCrossReportAnalysis extends ParentResult {

    private List<ImportCrossAnalysisResponse> content;

    public List<ImportCrossAnalysisResponse> getContent() {
        return content;
    }

    public ImportCrossReportAnalysis setContent(List<ImportCrossAnalysisResponse> content) {
        this.content = content;
        return this;
    }

    @Override
    public String toString() {
        return JSON.toJSONString(this);
    }

}


@Data
class ImportCrossAnalysisResponse {


    @ApiModelProperty(value = "路口名称", example = "经十路（舜耕路-山大路）",position = 0)
    private String crossName;

    @ApiModelProperty(value = "本周优化集中时段分布情况", example = "7:00~8:00,12:00~13:30,17:30~18:40",position = 1)
    private String optimizeSpan;

    @ApiModelProperty(value = "本周溢出优化次数", example = "10次",position = 2)
    private String overFlowOptimizeCount;
    @ApiModelProperty(value = "本周溢出主要发生方向", example = "北出口、南出口",position = 3)
    private String overFlowDirs;

    @ApiModelProperty(value = "本周拥堵优化次数", example = "10次",position = 4)
    private String congestOptimizeCount;
    @ApiModelProperty(value = "本周拥堵主要发生方向", example = "北出口、南出口",position = 5)
    private String congestDirs;

    @ApiModelProperty(value = "本周空放优化次数", example = "10次",position = 6)
    private String emptyPassOptimizeCount;
    @ApiModelProperty(value = "本周空放主要发生方向", example = "北出口、南出口",position = 7)
    private String emptyPassDirs;

    @ApiModelProperty(value = "本周失衡优化次数", example = "10次",position = 8)
    private String unbalanceOptimizeCount;
    @ApiModelProperty(value = "本周失衡主要发生方向", example = "北出口、南出口",position = 9)
    private String unbalanceDirs;
    //================================================================================================//
    @ApiModelProperty(value = "本周平均通行能力", example = "7800辆/小时",position = 10)
    private String avgCapacity;
    @ApiModelProperty(value = "平均通行能力与上周相比情况", example = "提升",position = 11)
    private String avgCapacityCompareSituation;
    //================================================================================================//
    @ApiModelProperty(value = "本周延误时间", example = "22",position = 12)
    private String delayTime;
    @ApiModelProperty(value = "延误时间与上周相比情况", example = "增加【减少】10%",position = 13)
    private String delayTimeCompare;
    //================================================================================================//
    @ApiModelProperty(value = "本周停车次数", example = "22",position = 14)
    private String avgStopTimes;
    @ApiModelProperty(value = "平均拥堵指数与上周相比情况", example = "增加【减少】10%",position = 15)
    private String avgCongestIndexCompare;
    //================================================================================================//
    @ApiModelProperty(value = "本周最大排队长度", example = "203",position = 16)
    private String maxQueueLength;
    @ApiModelProperty(value = "本周最大排队长度与上周相比情况", example = "提升【降低】10%",position = 17)
    private String maxQueueLengthCompare;
    //================================================================================================//
    @ApiModelProperty(value = "本周总体效果情况", example = "明显提升",position = 18)
    private String weekTotalitySituation;

    private List<ImportCrossReportAnalysisData> dataList = new ArrayList<>();

}


@Data
class ImportCrossReportAnalysisData {

    @ApiModelProperty(value = "星期", example = "星期一", position = 0)
    private String weekName;

    @ApiModelProperty(value = "策略优化时段", example = "7:35~8:40", position = 1)
    private String optimizeTimeSpan;

    @ApiModelProperty(value = "本周优化次数与问题次数比", example = "20/100", position = 2)
    private String thisWeekCountRadio;
    @ApiModelProperty(value = "上周优化次数与问题次数比", example = "20/100", position = 3)
    private String lastWeekCountRadio;

    @ApiModelProperty(value = "本周通行能力", example = "3466", position = 4)
    private Integer thisWeekCapacity;
    @ApiModelProperty(value = "上周通行能力", example = "5600", position = 5)
    private Integer lastWeekCapacity;

    @ApiModelProperty(value = "本周停车次数", example = "20", position = 6)
    private Double thisWeekStopTimes;
    @ApiModelProperty(value = "上周停车次数", example = "30", position = 7)
    private Double lastWeekStopTimes;

    @ApiModelProperty(value = "本周延误时间", example = "30.5", position = 8)
    private Double thisWeekDelayTime;
    @ApiModelProperty(value = "上周延误时间", example = "30.2", position = 9)
    private Double lastWeekDelayTime;

    @ApiModelProperty(value = "本周最大排队长度", example = "6.5", position = 10)
    private Double thisWeekMaxQueueLength;
    @ApiModelProperty(value = "上周最大排队长度", example = "-1.5%", position = 11)
    private Double lastWeekMaxQueueLength;

}



